@s HINSTANCE int
@s LPSTR int
\def\Math{{\it Mathematica\/ }}
\def\ML{{\it MathLink\/ }}
@* OpenGL/GLUT based viewer for \Math.
@d BANNER "GL Viewer for Mathematica"
@d VERSION "1.1"
@c
#ifdef __WIN32__
#  define ARCH "Win32 with OpenGL"
#elif defined __linux__
#  define ARCH "Linux 2.x with MesaGL 2.1"
#elif !defined(ARCH)
#  define ARCH "Solaris 2.51 with OpenGL"
#endif

@<Header@>@;
#ifndef MAX
#  define MAX(a,b) ((a)>(b) ? (a) : (b))
#endif
#ifndef MIN
#  define MIN(a,b) ((a)<(b) ? (a) : (b))
#endif
#define MAX3(a,b,c) ((a)>(b) ? MAX(a,c) : MAX(b,c))

@<Data structures@>@;
void CreateObject(void);
@<Global values@>@;
@<Function prototypes@>@;
@<Vector operations in cartesian cordinates@>@;
@<Memory management@>@;
@<Utilities@>@;
@<Read ThreeSript files@>@;
@<Average the normal vectors for the points@>@;
@<Count the length of the polygon/line list@>@;
@<Display the data structure with OpenGL@>@;
@<\Math-like printing@>@;
@<Trackball function@>@;
@<Write POVray file@>@;
@<Stuff to write a DXF file@>@;
@<Display function@>@;
@<Animation function for spin@>@;
@<Mouse function call back@>@;
@<Mouse motion call back@>@;
@<New reshape function to store the window size@>@;
@<Handle events inside the File menu@>@;
@<Handle events inside the Spin menu @>@;
@<Handle events inside of the Appearence menu@>@;
@<Handle events from the main menu@>@;
@<Create the main menu@>@;
@<Exit function@>@;
@<\ML-Functions@>@;
@<Main@>@;
@<Templates for \.{mprep}@>


@* The display. This function will be responsible for the redrawing of the objects
  in the scene.
@d TWO_PI (2.0*3.14159265358979324)
@d PI 3.14159265358979324
@<Display function@>=
void MVDisplay(void) {

   if(MakeNewView) {
     glPopMatrix();
     glPushMatrix();
     glTranslated(ZoomMove.x,ZoomMove.y,ZoomMove.z);
     MakeNewView=GL_FALSE;
    }
   if(MakeNewModel) {
     CreateObject();
     MakeNewModel=GL_FALSE;
    }
   if (DoReset) {
     glPopMatrix(); /* get the old matrix */
     glPushMatrix(); /* restore it for the next call */
     glTranslated(ZoomMove.x+CenterOfScene.x,
          ZoomMove.y+CenterOfScene.y,
          ZoomMove.z+CenterOfScene.z);
     if(DoTracking)
       glRotated(TrackAngle*ToDegree, TrackAxis.x, TrackAxis.y, TrackAxis.z);
     glTranslated(-CenterOfScene.x,-CenterOfScene.y,-CenterOfScene.z);
     if(DoReset) DoReset=GL_FALSE;
   }
  else if(DoTracking) {
    if(!DoTrackSpin && GetMouseTrack) {
      MVTrackball(&TrackAngle,&TrackAxis,ScreenXStart,ScreenYStart,ScreenX,ScreenY);
      DTrackAngle=MIN(TWO_PI/90.0,TrackAngle);
     }
    else
      TrackAngle+=DTrackAngle;
   }
  if(TrackAngle) {
    @<Do the rotation arround the track angle@>@;
   }
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  if(ObjID>0)
    glCallList(ObjID);
  glutSwapBuffers();
}

@ @<Do the rotation arround the track angle@>=
     glPopMatrix(); /* get the old matrix */
     glPushMatrix(); /* restore it for the next call */
     glTranslated(ZoomMove.x+CenterOfScene.x,
          ZoomMove.y+CenterOfScene.y,
          ZoomMove.z+CenterOfScene.z);
     glRotated(TrackAngle*ToDegree, TrackAxis.x, TrackAxis.y, TrackAxis.z);
     glTranslated(-CenterOfScene.x,-CenterOfScene.y,-CenterOfScene.z);

@ @<Animation function for spin@>=
void MVAnimate(void)
{
  if(MathLinkMode && MathLinkActive) {
    if (!ProcessMathLink(&LinkToMath)) exit(0);
   }
  if(DoTracking)
    glutPostRedisplay();
}


@ @<Mouse function call back@>=
void MVMouseButton(int btn, int state, int x, int y)
{
  if(btn != GLUT_LEFT_BUTTON || !DoTracking) return;
  switch(state) {
    case GLUT_DOWN :
      ScreenXStart=ScreenX=x;
      ScreenYStart=ScreenY=y;
      DoTrackSpin=GL_FALSE;
      break;
    case GLUT_UP :
      ScreenX=x;
      ScreenY=y;
      DoTrackSpin=GL_TRUE;
      break;
   }
}

@ @<Mouse motion call back@>=
void MVMouseMove(int x, int y)
{
  if(!DoTracking) return;
  ScreenX=x;
  ScreenY=y;
  glutPostRedisplay();
}

@ @<New reshape function to store the window size@>=
void MVReshape(int w, int h)
{
  glViewport(0,0,w,h);
  ScreenWidth=w;
  ScreenHeight=h;
}


@ @<Trackball function@>=
void MVTrackball(double *angle,
         Point3D *axis,
         int x1, int y1,
         int x2, int y2)
{ double phi1,theta1,phi2,theta2,n;
  Point3D v1,v2;

  phi1=PI*(2.0*x1-ScreenWidth)/((double) ScreenWidth);
  phi2=PI*(2.0*x2-ScreenWidth)/((double) ScreenWidth);
  theta1=PI*(ScreenHeight-y1)/((double) ScreenHeight);
  theta2=PI*(ScreenHeight-y2)/((double) ScreenHeight);
  CartesianFromSpherical(1.0,theta1,phi1,&v1);
  CartesianFromSpherical(1.0,theta2,phi2,&v2);
  Cross(axis,&v2,&v1);
  n=Norm(axis);
  if(n>0.0) {
    axis->x/=n;
    axis->y/=n;
    axis->z/=n;
   }
  else {
    axis->x=axis->y=0.0;
    axis->z=1.0;
   }
  *angle=acos(Dot(&v1,&v2));
}

@* Commandline interface.

@ @<Create the command line options@>=
  cmdln=DeclareCmdArg("mvfile",CMD_STRING,&ThreeScriptName,"       3Script input file.");
  cmdln=AppendCmdArg(DeclareCmdArg("mvs",CMD_SWITCH,&SmoothShading,
           "shows smooth/flat input data"),cmdln);
  cmdln=AppendCmdArg(DeclareCmdArg("mvw",CMD_SWITCH,&WireFrame,
           "show data as WireFrame"),cmdln);
  cmdln=AppendCmdArg(DeclareCmdArg("mvo",CMD_SWITCH,&PolyOutline,
           "show polygon outline"),cmdln);
  cmdln=AppendCmdArg(DeclareCmdArg("mvct",CMD_SWITCH,&COMTriangulation,
           "triangulate n-gons (n>4) with a new point in the center"),cmdln);

  cmdln=AppendCmdArg(DeclareCmdArg("mvts",CMD_DOUBLE,&LineTubeSize,
           "tube size of lines -- if given lines appear as tubes"),cmdln);
  cmdln=AppendCmdArg(DeclareCmdArg("mvta",CMD_DOUBLE,&LineSegmentAngle,
           "maximum angle for smooth line continuation"),cmdln);
  cmdln=AppendCmdArg(DeclareCmdArg("mvps",CMD_DOUBLE,&PointSphereSize,
           "sphere size of points -- if given points appear as spheres"),cmdln);
  cmdln=AppendCmdArg(
         DeclareCmdArg("mvpov",CMD_STRING,&POVOutName,"       POVRay output file."),
         cmdln);
  cmdln=AppendCmdArg(
         DeclareCmdArg("mvdxf",CMD_STRING,&DXFOutName,"       DXF output file."),
         cmdln);
  cmdln=AppendCmdArg(DeclareCmdArg("mvno",CMD_SWITCH,&NoDisplay,
           "run as filter and do not display the 3Script"),cmdln);

@ @<Evaluate the command line options@>=
   if (argc<=1) {
     HelpCmdArg(BANNER,cmdln);
     return 1;
    }
   cmdln=ScanCommandline(cmdln,argc,argv);
   if(*ThreeScriptName) {
     Scan3ScriptFile(ThreeScriptName);
     AveragedNormals(Polygons);
    }
   @<Calculate the $\cos$ of |LineSegmentAngle|@>@;
   MathLinkMode=IsMathLinkCall(argc,argv);
   @<Initialise the \ML-connection@>@;
   if(*ThreeScriptName && *POVOutName)
     WritePOVray(POVOutName,SmoothShading);
   if(*ThreeScriptName && *DXFOutName)
     WriteDXF(DXFOutName);
   if(NoDisplay) {
     cmdln=RemoveCmdArgs(cmdln);
     return 0;
    }
       
@ @<Calculate the $\cos$ of |LineSegmentAngle|@>=
   CosSkipCont=cos(ToRadian*(fabs(LineSegmentAngle)<MaxLSAngle? LineSegmentAngle : MaxLSAngle));

@ @<Global values@>=
char ThreeScriptName[256]="";
int SmoothShading=0;
int WireFrame=GL_FALSE;
int PolyOutline=GL_FALSE;
int COMTriangulation=GL_FALSE; /* triangulate with a new point in the center */
Point3D ZoomMove={0.,0.,0.};
Point3D CenterOfScene={0.0,0.0,0.0};
Point3D ScaleOfScene={1.0,1.0,1.0};
double MaxScaleOfScene=1.0;
int MakeNewView=GL_FALSE;
double RadialZoom=1.0;

@ Output file names to run as filter.
@<Global values@>=
char POVOutName[256]="";
char DXFOutName[256]="";
int NoDisplay=GL_FALSE;

@ |LineTubeSize| controls the appearence of lines. If it is unequal to 0 lines are drawn as tubes.
  Simmilar to |LineTubeSize| the variable |PointSphereSize| controls the
  appearence of points. If |PointSphereSize| is greather than zero points are
  drawn as spheres of that size.
  When lines are continued we will only use smooth continuation for
  line segments with an angle less than |LineSegmentAngle|. The maximum allowed
  angle where continuation is possible is |MaxLSAngle|.
@d MaxLSAngle (75.0)
@<Global values@>=
double LineTubeSize=0.0;
int TubeSegments=12;
double LineSegmentAngle=30.; /* degree */
double CosSkipCont;
double PointSphereSize=0.0;
const RGBColor Black={0.0,0.0,0.0};
const RGBColor White={1.0,1.0,1.0};
Point3D XAxis={1.0,0.0,0.0};
Point3D YAxis={0.0,1.0,0.0};
Point3D ZAxis={0.0,0.0,1.0};

@ @<Global values@>=
int MakeNewModel=GL_FALSE;
int DoReset=GL_FALSE;
int DoTracking=GL_FALSE;
int DoTrackSpin=GL_FALSE;
int GetMouseTrack=GL_TRUE;
int ScreenX,ScreenY;
int ScreenXStart,ScreenYStart;
Point3D TrackAxis;
double TrackAngle,DTrackAngle;
int ScreenWidth,ScreenHeight;

@* The menu initialisation.

@d MENU_ID_SAVEPOVRAY_SMOOTH   (101)
@d MENU_ID_SAVEPOVRAY_FLAT     (102)
@d MENU_ID_SAVE_DXF            (103)


@d MENU_ID_SPIN_X     (201)
@d MENU_ID_SPIN_Y     (202)
@d MENU_ID_SPIN_Z     (203)
@d MENU_ID_SPIN_SLOW  (204)
@d MENU_ID_SPIN_FAST  (205)
@d MENU_ID_SPIN_TRACKBALL  (206)
@d MENU_ID_STOP       (207)


@d MENU_ID_SHOW_SMOOTH (301)
@d MENU_ID_SHOW_FLAT   (302)
@d MENU_ID_SHOW_WIRE   (303)

@d MENU_ID_ABOUT      (401)
@d MENU_ID_RESET      (402)
@d MENU_ID_FLIP_FACE  (403)
@d MENU_ID_ZOOM_IN    (404)
@d MENU_ID_ZOOM_OUT   (405)
@d MENU_ID_CLEAR      (406)
@d MENU_ID_QUIT       (666)

@<Global values@>=
int menuIDMain;
int menuIDFiles;
int menuIDSpin;
int menuIDAppear;

@ Create the main menu and attatch it to the right mouse button
@<Create the main menu@>=
void MVCreateMainMenu(void) {

   @<Submenu for files@>@;
   @<Submenu for spin of the object@>@;
   @<Submenu for the appearence of the object@>@;
   @<Submenu for the appearence of the object@>@;
   menuIDMain=glutCreateMenu(MVWinMainMenu);
   glutAddSubMenu("Files...",menuIDFiles);
   glutAddSubMenu("Spin...",menuIDSpin);
   glutAddSubMenu("Appearence ...",menuIDAppear);
   glutAddMenuEntry("Reset view",MENU_ID_RESET);
   glutAddMenuEntry("Zoom out",MENU_ID_ZOOM_OUT);
   glutAddMenuEntry("Zoom in",MENU_ID_ZOOM_IN);
   glutAddMenuEntry("Flip faces",MENU_ID_FLIP_FACE);
   if(! MathLinkMode) {
     glutAddMenuEntry("About",MENU_ID_ABOUT);
     glutAddMenuEntry("Quit",MENU_ID_QUIT);
    }
   else
     glutAddMenuEntry("Clear Window",MENU_ID_CLEAR);

   glutAttachMenu(GLUT_RIGHT_BUTTON);
  }

@ Event handling in the main menu.
@<Handle events from the main menu@>=
void MVWinMainMenu(int what)
{ double vr,vtheta,vphi;

   switch(what) {
     case MENU_ID_RESET:
       @<Reset the global parameter@>@;
       break;
     case MENU_ID_FLIP_FACE :
       ReversePolygons(Polygons);
       AveragedNormals(Polygons);
       MakeNewModel=GL_TRUE;
       break;
     case MENU_ID_ZOOM_OUT :
       RadialZoom*=1.5;
       @<Recalculate the zoom translation@>@;
       break;
     case MENU_ID_ZOOM_IN :
       RadialZoom/=1.5;
       @<Recalculate the zoom translation@>@;
       break;
     case MENU_ID_ABOUT:
       printf(BANNER
          " (c) Jens-Peer Kuska  Version " VERSION    @|
          "\nfor " ARCH "\ncompiled at "__DATE__ "\n");
       if(*ThreeScriptName)
         printf("3Script file   : %s\n",ThreeScriptName);
       printf("Screen         : %d x %d\n",ScreenWidth,ScreenHeight);
       printf("Center         : {%lg, %lg, %lg}\n",
          CenterOfScene.x,CenterOfScene.y,CenterOfScene.z);
       printf("Scaling        : {%lg, %lg, %lg}\n",
          ScaleOfScene.x,ScaleOfScene.y,ScaleOfScene.z);
       if(ObjID>0)
         printf("Compiled obj   : %d\n",ObjID);
       if(PoolSize)
         printf("Unique Points  : %d of %d\n",PoolSize,MaxPoolSize);
       printf("Polygon number : %ld\n",LengthLinePolyList(Polygons));
       printf("Line number    : %ld\n",LengthLinePolyList(Lines));
       printf("Polygon shading: %s\n",SmoothShading ?  "smooth" : "flat" );
       printf("Wire frame     : %s\n",WireFrame ? "On" : "Off");
       printf("Tracking mode  : %s\n",DoTracking ? "on" : "off");
       break;
     case MENU_ID_CLEAR:
       mvClearContents();
       break;
     case MENU_ID_QUIT :
       exit(0);
       break;
    }
   glutPostRedisplay();
}

@ @<Reset the global parameter@>=
 DoReset=GL_TRUE;
 DoTracking=DoTrackSpin=GL_FALSE;
 ZoomMove.x=ZoomMove.y=ZoomMove.z=0.0;
 RadialZoom=1.0;
 TrackAngle=0.0;
 TrackAxis.x=TrackAxis.y=0.0;
 TrackAxis.z=1.0;
 ScreenXStart=ScreenX=0;
 ScreenYStart=ScreenY=0;

@ @<Recalculate the zoom translation@>=
       SphericalFromCartesian(&ViewPoint, &vr,  &vtheta, &vphi);
       ZoomMove.x=ViewPoint.x-RadialZoom*vr*cos(vphi)*sin(vtheta);
       ZoomMove.y=ViewPoint.y-RadialZoom*vr*sin(vphi)*sin(vtheta);
       ZoomMove.z=ViewPoint.z-RadialZoom*vr*cos(vtheta);

       MakeNewView=GL_TRUE;


@ Create the Files-menu.
@<Submenu for files@>=
menuIDFiles=glutCreateMenu(MVWinFileMenu);
glutAddMenuEntry("Save as smooth POVRay file",MENU_ID_SAVEPOVRAY_SMOOTH);
glutAddMenuEntry("Save as flat POVRay file",MENU_ID_SAVEPOVRAY_FLAT);
glutAddMenuEntry("Save DXF file",MENU_ID_SAVE_DXF);

@ @<Handle events inside the File menu@>=
void MVWinFileMenu(int what)
{
  switch(what) {
    case MENU_ID_SAVEPOVRAY_SMOOTH :
      WritePOVray("mview3ds.pov",GL_TRUE);
      break;
    case MENU_ID_SAVEPOVRAY_FLAT :
      WritePOVray("mview3df.pov",GL_FALSE);
      break;
    case MENU_ID_SAVE_DXF :
      WriteDXF("mview3d.dxf");
      break;
   }
  glutPostRedisplay();
}

@ @<Submenu for spin of the object@>=
menuIDSpin=glutCreateMenu(MVWinSpinMenu);
glutAddMenuEntry("Trackball",MENU_ID_SPIN_TRACKBALL);
glutAddMenuEntry("X axes spin",MENU_ID_SPIN_X);
glutAddMenuEntry("Y axes spin",MENU_ID_SPIN_Y);
glutAddMenuEntry("Z axes spin",MENU_ID_SPIN_Z);
glutAddMenuEntry("Spin slower",MENU_ID_SPIN_SLOW);
glutAddMenuEntry("Spin faster",MENU_ID_SPIN_FAST);
glutAddMenuEntry("Halt",MENU_ID_STOP);




@ @<Handle events inside the Spin menu @>=
void MVWinSpinMenu(int what)
{
  switch(what) {
    case MENU_ID_SPIN_TRACKBALL :
      GetMouseTrack=GL_TRUE;
      @<Switch the tracking mode@>@;
      break;

    case MENU_ID_SPIN_X :
      TrackAxis.x=1.0; @+ TrackAxis.y=TrackAxis.z=0.0;
      @<Set spin parameter for a fixed axis@>@;
      break;
    case MENU_ID_SPIN_Y :
      TrackAxis.y=1.0; @+ TrackAxis.x=TrackAxis.z=0.0;
      @<Set spin parameter for a fixed axis@>@;
      break;
    case MENU_ID_SPIN_Z :
      TrackAxis.z=1.0; @+ TrackAxis.x=TrackAxis.y=0.0;
      @<Set spin parameter for a fixed axis@>@;
      break;
    case MENU_ID_SPIN_SLOW :
       DTrackAngle*=0.5;
      break;
    case MENU_ID_SPIN_FAST :
      DTrackAngle*=2.0;
      break;
    case MENU_ID_STOP :
      @<Stop spining by parameter reset@>@;
      break;
  }
  glutPostRedisplay();
}

@ @<Stop spining by parameter reset@>=
  DoTrackSpin=GL_FALSE;
  TrackAngle=0.0;
  DoTracking=GL_FALSE;

@ @<Switch the tracking mode@>=
 DoTracking =GL_TRUE;
 DoTrackSpin=GL_FALSE;

@ @<Set spin parameter for a fixed axis@>=
DoTrackSpin=DoTracking=GL_TRUE;
GetMouseTrack=GL_FALSE;
TrackAngle=0.0;
DTrackAngle=TWO_PI/80.;


@ @<Submenu for the appearence of the object@>=
menuIDAppear=glutCreateMenu(MVWinAppearMenu);
glutAddMenuEntry("Smooth surface",MENU_ID_SHOW_SMOOTH);
glutAddMenuEntry("Flat surface",MENU_ID_SHOW_FLAT);
glutAddMenuEntry("Wire frame",MENU_ID_SHOW_WIRE);

@ @<Handle events inside of the Appearence menu@>=
void MVWinAppearMenu(int what)
{
   switch(what) {
     case MENU_ID_SHOW_SMOOTH :
        SmoothShading=GL_TRUE;
        WireFrame=GL_FALSE;
        break;
     case MENU_ID_SHOW_FLAT :
        SmoothShading=GL_FALSE;
        WireFrame=GL_FALSE;
        break;
     case MENU_ID_SHOW_WIRE :
        WireFrame=GL_TRUE;
        break;
    } 
   MakeNewModel=GL_TRUE;
   glutPostRedisplay();
}


@ The |exit| function.
@<Exit function@>=
void MathViewExit(void)
{ int i;
  if (PointPool) free(PointPool);
  if(NormalPool)
    for(i=0; i<PoolSize; i++)
      if(NormalPool[i]) free(NormalPool[i]);
  free(NormalPool);
  Points=RemovePointList(Points);
  Lines=RemoveLinePolyList(Lines);
  Polygons=RemoveLinePolyList(Polygons);
  if(MathLinkMode) {
    @<Close \ML-connection@>@;
   }
}

@* The main function.
@ This step is for Microsofts OpenGL 1.1. The libraries
   assume that floating point exceptions are masked, just
   as they are by default with Microsoft tools. Borland
   development tools leave the X86 unmasked by default.
@ @<Mask floating point exceptions@>=
#if defined(__WIN32__) && defined(__BORLANDC__)
   _control87(MCW_EM,MCW_EM);
#endif


@ @<Main@>=
#ifdef __WIN32__
#pragma argsused
int PASCAL WinMain(HINSTANCE hCurInstance,
                   HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow) {
char *argv[32];
int argc,i;
char **argv_end=argv+32;
char buff[512];
char *buff_start=buff;
#else
int main(int argc, char *argv[]) {
#endif
   CmdArgLst *cmdln;
   long int err;
   void MathViewExit(void);
#ifdef __WIN32__
    MLScanString(argv,&argv_end,&lpCmdLine,&buff_start);
    argc=argv_end-argv;
    for(i=argc; i>0; i--)
      argv[i]=argv[i-1];
    argv[0]="mathview3d";
    argc++;
#endif
   @<Create the command line options@>@;
   atexit(MathViewExit);
   @<Evaluate the command line options@>@;
   @<Mask floating point exceptions@>@;
   glutInit(&argc,argv);
   cmdln=RemoveCmdArgs(cmdln);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow(BANNER);
   MVCreateMainMenu();
   glutMouseFunc(MVMouseButton);
   glutMotionFunc(MVMouseMove);
   glutDisplayFunc(MVDisplay);
   glutReshapeFunc(MVReshape);
   glutIdleFunc(MVAnimate);
   InitView();
   glutMainLoop();
   return 0;
 }

@* Datastructures.
@<Data structures@>=
@<3D Point@>@;
@<RGB color@>@;
@<List of Points@>@;
@<List of Lines or Polygons@>@;

@ @<3D Point@>=
typedef struct Point3D { double x,y,z; } Point3D;

@ The coordinates of all points in the scene will be stored in |PointPool|.
  If one of the points belong to a surface, than the corresponding entry in
  |NormalPool[i]| is unequal to |NULL| and will store a averaged normal vector.

@<Global values@>=
Point3D *PointPool=NULL;
Point3D **NormalPool=NULL;
int PoolSize=0;
int MaxPoolSize=0;

@ @<\Math like print of points@>=
void MathPrintPosition(FILE *f, int index)
{ Point3D *p;

  p=PointPool+index;
  fprintf(f,"{ %lg, %lg, %lg}",p->x,p->y,p->z);
}

@ @<\Math-like printing@>=
@<\Math like print of points@>@;


@ @<RGB color@>=
typedef struct RGBColor { double r,g,b; } RGBColor;

@ @<List of Points@>=
typedef struct PointList {
               RGBColor color;
               double size;
               int point;
               struct PointList *next;
         } PointList;

@ @<New point in the point list@>=
PointList *PrependPointList(PointList *lst, int index,
                 double size,
                 double red, double green, double blue)
{ PointList *head;

  head=(PointList *)malloc(sizeof(PointList));
  head->next=lst;
  head->point=index;
  head->size=size;
  head->color.r=red;
  head->color.g=green;
  head->color.b=blue;
  return head;
}

@ @<Remove point list@>=
PointList *RemovePointList(PointList *lst)
{ PointList *head;

  while(lst) {
    head=lst;
    lst=lst->next;
    free(head);
   }
  return NULL;
}

@ @<Memory management@>=
@<New point in the point list@>@;
@<Remove point list@>@;
@ @<\Math like print of point lists@>=
void MathPrintPointList(FILE *f, PointList *lst)
{
  fprintf(f,"{");
  while(lst) {
     fprintf(f,"\t{PointSize[%lg], RGBColor[%lg, %lg, %lg], Point[",
        lst->size,
        lst->color.r,
        lst->color.g,
        lst->color.b);
     MathPrintPosition(f,lst->point);
     fprintf(f,"]}");
     if(lst->next) fprintf(f,",\n");
     lst=lst->next;
    }
  fprintf(f,"}");
 }

@ @<\Math-like printing@>=
@<\Math like print of point lists@>@;

@ @d Line3D (1) @d Polygon3D (2)
@<List of Lines or Polygons@>=
typedef struct LinePolyList {
                     short int what;
                     RGBColor color;
                     int    aux1;
                     double aux2;
                     int *points;
                     int pno;
                     struct LinePolyList *next;
                    } LinePolyList;

@ \Math produces polygones of more than three points.
 |PrependLinePolyList| will
  triangulate the incomming polygons at definition.
@<Prepend line or polygon to |lst|@>=
LinePolyList *PrependLinePolyList(LinePolyList *lst,
                     short int what,
                     int *lpoints,
                     int pno,
                     double r, double g, double b)
{ LinePolyList *head;
  int tri[3], *newpoints,nold;
  int i,j;
  
  if(lst && Line3D==what && lpoints[0]==lst->points[lst->pno-1]) {
    @<Glue continued lines to one and |return|@>@;
   }
  if(Polygon3D==what && pno>3) {
    if(COMTriangulation)
      @<Add a center point for triangulation of $n$-gons@>@;
    else
      @<Triangulate and store $n$-gons@>@;
   }
  head=(LinePolyList *)malloc(sizeof(LinePolyList));
  head->next=lst;
  head->what=what;
  head->pno=pno;
  head->points=(int *)calloc(pno,sizeof(int));
  /* copy line points and remove multiple succesive points */
  head->points[0]=lpoints[0];
  for(i=j=1; i<pno; i++)
    if(lpoints[i-1]!=lpoints[i])
      head->points[j++]=lpoints[i];
  if(j!=pno) head->pno=j;
  head->color.r=r;
  head->color.g=g;
  head->color.b=b;
  head->aux1=-1;
  head->aux2=-1.0;
  return head;
}

@ @<Remove a line polygon list@>=
LinePolyList *RemoveLinePolyList(LinePolyList *lst)
{ LinePolyList *head;

  while(lst) {
    free(lst->points);
    head=lst->next;
    free(lst);
    lst=head;
   }
  return NULL;
}

@ \Math's \.{ParametricPlot3D} has the habit to produce a sequence
  of single line segments of the pattern
  \.{\{\_\_\_, Line[\{a\_\_,b\_\}], Line[\{b\_,c\_\_\}], \_\_\_\}}.
  we will glue this lines to one long line.
@ @<Glue continued lines to one and |return|@>=
    nold=lst->pno;
    newpoints=(int *)calloc(nold+pno-1,sizeof(int));
    memcpy(newpoints,lst->points,nold*sizeof(int));
    memcpy(newpoints+nold,lpoints+1,(pno-1)*sizeof(int));
    free(lst->points);
    lst->points=newpoints;
    lst->pno += pno-1;
    return lst;


@ @<Triangulate and store $n$-gons@>=
{ int i; 

  head=PrependLinePolyList(lst,what,lpoints,3,r,g,b);
  for(i=1; i<pno-1; i++)
    lpoints[i]=lpoints[i+1];
  return PrependLinePolyList(head,what,lpoints,pno-1,r,g,b);
}

@ @<Add a center point for triangulation of $n$-gons@>=
{ Point3D cntr={0.0,0.0,0.0};
  int i;

  head=lst;
  /* calculate the center of the $n$-gon */
  for(i=0; i<pno; i++) {
    cntr.x+=PointPool[lpoints[i]].x;
    cntr.y+=PointPool[lpoints[i]].y;
    cntr.z+=PointPool[lpoints[i]].z;
   }
  cntr.x/=pno;
  cntr.y/=pno;
  cntr.z/=pno;
  /* store the center of the polygon as new point in the pool */
  tri[0]=AddToPointPool(cntr.x,cntr.y,cntr.z);
  /* now we make triangles from the center to two eges */
  for(i=1; i<pno; i++) {
    tri[1]=lpoints[i-1];
    tri[2]=lpoints[i];
    head=PrependLinePolyList(head,what,tri,3,r,g,b);
   }
  tri[1]=lpoints[pno-1];
  tri[2]=lpoints[0];
  return PrependLinePolyList(head,what,tri,3,r,g,b);
}

@ @<Count the length of the polygon/line list@>=
long int LengthLinePolyList(LinePolyList *lst)
{
  long int len=0;

  while(lst) {
    len++;
    lst=lst->next;
   }
  return len;
}

@ @<Reverse the order of points in the polygons@>=
void ReversePolygons(LinePolyList *lst)
{ int n,tmp;

  while(lst) {
    if(Polygon3D==lst->what) {
      n=lst->pno;
      if(3==n) {
         tmp=lst->points[2];
         lst->points[2]=lst->points[0];
        lst->points[0]=tmp;
       }
     }
    lst=lst->next;
   }
}


@ @<Utilities@>=
    @<Remove a line polygon list@>@;
    @<Reverse the order of points in the polygons@>@;

@ @<Memory management@>=
@<Prepend line or polygon to |lst|@>@;

@ @<\Math like print of polygons and lines@>=
void MathPrintLinePolyList(FILE *f, LinePolyList *lst)
{ int i;

  fprintf(f,"{");
  while(lst) {
     fprintf(f,"{\n\t%s[%lg, %lg, %lg]%s,\n\t%s[{",
         Line3D==lst->what ? "RGBColor" : "SurfaceColor[RGBColor[",
         lst->color.r,
         lst->color.g,
         lst->color.b,
         Line3D==lst->what ? "": "]",
         Line3D==lst->what ? "Line" : "Polygon");
     for(i=0; i<lst->pno; i++) {
        MathPrintPosition(f,lst->points[i]);
        if(i!= lst->pno - 1)
          fprintf(f,",");
      }
     fprintf(f,lst->next ? "}]},\n" : "}]}");
    }
  fprintf(f,"}\n");
}


@ @<\Math-like printing@>=
@<\Math like print of polygons and lines@>@;

@ @<Cyclic index |j1|, |j2| and |j3|@>=
  j1=lst->points[0==i ? lst->pno-1: i-1];
  j2=lst->points[i];
  j3=lst->points[lst->pno-1 == i ? 0 : i+1];

@ Calculate the averaged normals for the points in polygons.
@<Average the normal vectors for the points@>=
void AveragedNormals(LinePolyList *lst)
{ int i,j1,j2,j3;
  Point3D normal;
  int *cnt,res;
  double n;

  cnt=(int *)calloc(PoolSize,sizeof(int));
  for(i=0; i<PoolSize; i++) cnt[i]=0;
  while (lst) {
    for(i=0; i<lst->pno; i++) {
      @<Cyclic index |j1|, |j2| and |j3|@>@;
      res=SurfaceNormal(&normal,PointPool+j1,PointPool+j2,PointPool+j3);
      if(!res) continue;
      if(!NormalPool[j2]) {
        NormalPool[j2]=(Point3D *)malloc(sizeof(Point3D));
        NormalPool[j2]->x=NormalPool[j2]->y=NormalPool[j2]->z=0.0;
       }
      NormalPool[j2]->x+=normal.x;
      NormalPool[j2]->y+=normal.y;
      NormalPool[j2]->z+=normal.z;
      cnt[j2]++;
    }
   lst=lst->next;
  }
 for(i=0; i<PoolSize; i++)
   if(cnt[i] && NormalPool[i]) {
     NormalPool[i]->x/=cnt[i];
     NormalPool[i]->y/=cnt[i];
     NormalPool[i]->z/=cnt[i];
     n=Norm(NormalPool[i]);
     if(n>0.0) {
       NormalPool[i]->x/=n;
       NormalPool[i]->y/=n;
       NormalPool[i]->z/=n;
      }
     else {
       free(NormalPool[i]);
       NormalPool[i]=NULL;
      }
    }
 free(cnt);
}

@ @<Global values@>=
PointList    *Points=NULL;
LinePolyList *Lines=NULL;
LinePolyList *Polygons=NULL;

@ Here we allocate the storrage for the point pool and the pool of normals.
@d INIT_POOLSIZE 8192
@d POOL_INC 1024
@d TINY (0.1e-9)
@<Function prototypes@>=
int AddToPointPool(double x, double y, double z);
@ @<Add a new position to the |PointPool|@>=
int AddToPointPool(double x, double y, double z)
{ int pn,i;
  double d;
  Point3D *pos, **nvec;

  if (!PointPool) { /* first call */
    PointPool=(Point3D *)calloc(MaxPoolSize=INIT_POOLSIZE,sizeof(Point3D));
    NormalPool=(Point3D **)calloc(INIT_POOLSIZE,sizeof(Point3D *));
    for(i=0; i<MaxPoolSize; i++)
      NormalPool[i]=NULL;
    PoolSize=0;
   }
  pn=PoolSize;
  pos=PointPool;
  for(i=0; i<pn; i++,pos++) {
     d=fabs(x-pos->x)+fabs(y-pos->y)+fabs(z-pos->z);
     if(d<TINY) return i;
    }
  if(pn==MaxPoolSize) { /* add |POS_INC| points to the point pool */
     pos=(Point3D *) calloc(MaxPoolSize+POOL_INC,sizeof(Point3D));
     memcpy(pos,PointPool,MaxPoolSize*sizeof(Point3D));
     free(PointPool);
     PointPool=pos;
     nvec=(Point3D **)calloc(MaxPoolSize+POOL_INC,sizeof(Point3D *));
     memcpy(nvec,NormalPool,MaxPoolSize*sizeof(Point3D*));
     free(NormalPool);
     NormalPool=nvec;
     MaxPoolSize+=POOL_INC;
         for(i=PoolSize; i<MaxPoolSize; i++)
           NormalPool[i]=NULL;
    }
    /* insert the new point */
  PointPool[pn].x=x;
  PointPool[pn].y=y;
  PointPool[pn].z=z;
  @<Update the bounding box@>@;
  return PoolSize++;
}

@ @<Update the bounding box@>=
  if(BoundP1.x>x) BoundP1.x=x;
  if(BoundP2.x<x) BoundP2.x=x;
  if(BoundP1.y>y) BoundP1.y=y;
  if(BoundP2.y<y) BoundP2.y=y;
  if(BoundP1.z>z) BoundP1.z=z;
  if(BoundP2.z<z) BoundP2.z=z;


                    
@ @<Memory management@>=
@<Add a new position to the |PointPool|@>@;

@ @d MAX_LIGHT (8)
@<Global values@>=
Point3D ViewPoint;
RGBColor AmbientColor;
Point3D BoundP1,BoundP2;
Point3D LightPosition[MAX_LIGHT];
RGBColor   LightColor[MAX_LIGHT];
int LightNo=0;
int PolygonEdge=1;

@* Vector operations.
@<Vector operations in cartesian cordinates@>=
@<Dot product@>@;
@<Cross product@>@;
@<Norm of a vector@>@;
@<Spherical coordinates@>@;
@<Cartesian from spherical@>@;
@<Surface normal@>@;
@<Find the two other unit vectors for $v_1$@>@;
@<Gram Schmidt orthogonalization@>@;
@<Calculate a point |p| on the cone@>@;

@ @<Dot product@>=
double Dot(Point3D *p1, Point3D *p2)
{
  return p1->x * p2->x + p1->y * p2->y + p1->z * p2->z;
}

@ @<Cross product@>=
void Cross(Point3D *c, Point3D *a, Point3D *b)
{
 c->x=-a->z * b->y+a->y * b->z;
 c->y= a->z * b->x-a->x * b->z;
 c->z=-a->y * b->x+a->x * b->y;
}

@ @<Norm of a vector@>=
double Norm(Point3D *p)
{
  return sqrt(Dot(p,p));
}

@ @<Surface normal@>=
int SurfaceNormal(Point3D *nrm,
       Point3D *p1, Point3D *p2, Point3D *p3)
{ Point3D v1,v2;
  double n;

  v1.x=p2->x-p1->x;
  v1.y=p2->y-p1->y;
  v1.z=p2->z-p1->z;
  v2.x=p3->x-p2->x;
  v2.y=p3->y-p2->y;
  v2.z=p3->z-p2->z;
  Cross(nrm,&v1,&v2);
  n=Norm(nrm);
  if(n!=0.0) {
    nrm->x/=n;
    nrm->y/=n;
    nrm->z/=n;
    return GL_TRUE;
   }
  return GL_FALSE;

}

@ @<Spherical coordinates@>=
void SphericalFromCartesian(Point3D *p,
       double *r, double *theta, double *phi)
{
  *r=Norm(p);
  if(p->x !=0.0 && p->y !=0.0)
    *phi=atan2(p->y,p->x);
  else
    *phi=0.0;
  if(*r==0.0)
    *theta=0.0;
  else
    *theta=acos(p->z/(*r));
}

@ @<Cartesian from spherical@>=
void CartesianFromSpherical(double r, double theta, double phi,
               Point3D *p)
{ double st;

  st=r*sin(theta);
  p->x=cos(phi)*st;
  p->y=sin(phi)*st;
  p->z=r*cos(theta);
}

     
@ To draw lines as cones we need for a normalised vector $v_1$ the two
  orthogonal vectors $v_2$ and $v_3$.
@d sign(a) (a>0.0 ? 1. : -1.)

@<Find the two other unit vectors for $v_1$@>=
void OrthogonalSet(Point3D *v1, Point3D *v2, Point3D *v3)
{ double a,b,c,n,*p;
  int i;

  a=v1->x; @+ b=v1->y; @+ c=v1->z;
  p= (double *) v1;
  i=0;
  if(fabs(p[i])<fabs(p[1])) i=1;
  if(fabs(p[i])<fabs(p[2])) i=2;
  switch (i) {
    case 0 : 
      @<Calcultate $v_2$ und $v_3$ for maximal x-component in $v_1$@>@;
      break;
    case 1 :
      @<Calcultate $v_2$ und $v_3$ for maximal y-component in $v_1$@>@;
      break;
    case 2 :
      @<Calcultate $v_2$ und $v_3$ for maximal z-component in $v_1$@>@;
      break;
  }
}

@ @<Calcultate $v_2$ und $v_3$ for maximal z-component in $v_1$@>=     
n=sqrt(b*b+c*c);
if(n>0.0) {
  v2->x=n;
  v2->y=-a*b/n;
  v2->z=-a*c/n;

  v3->x=0.0;
  v3->y=fabs(c)/n;
  v3->z=- b*sign(c)/n;
}

@ @<Calcultate $v_2$ und $v_3$ for maximal x-component in $v_1$@>=
n=sqrt(a*a+c*c);
if(n>0.0) {
  v2->x=- a*b/n,
  v2->y= n;
  v2->z= - b*c/n;

  v3->x=- c*sign(a)/n;
  v3->y=0;
  v3->z=fabs(a)/n;
 }

@ @<Calcultate $v_2$ und $v_3$ for maximal y-component in $v_1$@>=
n=sqrt(b*b+c*c);
if(n>0.0 ) {
  v2->x=n;
  v2->y=-a*b/n;
  v2->z=-a*c/n;

  v3->x=0.0;
  v3->y=-c*sign(b)/n;
  v3->z=fabs(b)/n;
 } 

@ @<Gram Schmidt orthogonalization@>=
void GramSchmidt(Point3D *e1, Point3D *e2, Point3D *e3)
  { double d12,d13,d23, n;

    d12=Dot(e1,e2);
    e2->x -= d12 * e1->x;
    e2->y -= d12 * e1->y;
    e2->z -= d12 * e1->z;
    n=Norm(e2);
    e2->x/=n; @+    e2->y/=n; @+    e2->z/=n;
    d13=Dot(e1,e3);
    e3->x -= d13 * e1->x;
    e3->y -= d13 * e1->y;
    e3->z -= d13 * e1->z;
    n=Norm(e3);
    e3->x/=n; @+    e3->y/=n; @+    e3->z/=n;
    d23=Dot(e2,e3);
    e3->x -= d23 * e2->x;
    e3->y -= d23 * e2->y;
    e3->z -= d23 * e2->z;
    n=Norm(e3);
    e3->x/=n; @+    e3->y/=n; @+    e3->z/=n;
  }



@ @d sqr(a) ((a)*(a)) 
@<Calculate a point |p| on the cone@>=
void ConePoint(double r, double phi, Point3D *u, Point3D *v, Point3D *p)
{ double cphi,sphi;

  cphi=r*cos(phi);
  sphi=r*sin(phi);
  p->x=(cphi*u->x+sphi*v->x);
  p->y=(cphi*u->y+sphi*v->y);
  p->z=(cphi*u->z+sphi*v->z);

}


@ @d IncPoint3D(p,inc) {p.x+=inc.x; p.y+=inc.y; p.z+=inc.z;} 
  @d AddPoint3D(res,a,b) {res.x=a.x+b.x; res.y=a.y+b.y; res.z=a.z+b.z; }

@<Make a cone along $p_1$ and $p_2$@>=
void MakeCone(Point3D *p1, Point3D *p2, double r, int n)
{ Point3D v1,v2,v3,vn,p[4];
  double norm;
  double dphi;
  int i;

  v1.x=p2->x-p1->x;
  v1.y=p2->y-p1->y;
  v1.z=p2->z-p1->z;
  norm=Norm(&v1);
  v1.x/=norm; @+  v1.y/=norm; @+ v1.z/=norm;
  OrthogonalSet(&v1,&v2,&v3);

  v1.x=p2->x-p1->x;
  v1.y=p2->y-p1->y;
  v1.z=p2->z-p1->z;

  dphi=TWO_PI/n;
  ConePoint(r,0.0,&v2,&v3, p);
  vn=*p;
  IncPoint3D(p[0],(*p1))@;
  AddPoint3D(p[1],p[0],v1)@;
  glBegin(GL_TRIANGLE_STRIP);
    glVertex3d(p[0].x,p[0].y,p[0].z);
    glNormal3d(vn.x,vn.y,vn.z);
    glVertex3d(p[1].x,p[1].y,p[1].z);
    glNormal3d(vn.x,vn.y,vn.z);
    for(i=1; i<=n; i++) {
      ConePoint(r,i*dphi,&v2,&v3,p+2);
      vn=p[2];
      IncPoint3D(p[2],(*p1))@;
      AddPoint3D(p[3],p[2],v1)@;
      glVertex3d(p[2].x,p[2].y,p[2].z);
      glNormal3d(vn.x,vn.y,vn.z);

      glVertex3d(p[3].x,p[3].y,p[3].z);
      glNormal3d(vn.x,vn.y,vn.z);

    }
  glEnd();

}

@ @<Make a continued cone along $p_1$ and $p_2$ to $p_3$@>=
void MakeConeContinued(Point3D *p1, Point3D *p2, Point3D *p3,
               double r, int n)
{ Point3D v1,v2,v3,u1,u2,u3,vn1,vn2,p[4];
  double norm;
  double dphi;
  int i;

  v1.x=p2->x-p1->x; @+ v1.y=p2->y-p1->y; @+ v1.z=p2->z-p1->z;
  norm=Norm(&v1);
  v1.x/=norm; @+  v1.y/=norm; @+ v1.z/=norm;
  OrthogonalSet(&v1,&v2,&v3);
  u1.x=p3->x-p2->x;@+  u1.y=p3->y-p2->y; @+  u1.z=p3->z-p2->z;
  norm=Norm(&u1);
  u1.x/=norm; @+  u1.y/=norm; @+ u1.z/=norm;
  @<Skip coninuation if the angle between the vectors is to large@>@;
  u2=v2; @+  u3=v3;
  GramSchmidt(&u1,&u2,&u3);
  dphi=TWO_PI/n;
  ConePoint(r,0.0,&v2,&v3, p);
  vn1=*p;
  IncPoint3D(p[0],(*p1))@;
  ConePoint(r,0.0,&u2,&u3,p+1);
  vn2=p[1];
  IncPoint3D(p[1],(*p2))
  glBegin(GL_TRIANGLE_STRIP);
    glVertex3d(p[0].x,p[0].y,p[0].z);
    glNormal3d(vn1.x,vn1.y,vn1.z);
    glVertex3d(p[1].x,p[1].y,p[1].z);
    glNormal3d(vn2.x,vn2.y,vn2.z);
    for(i=1; i<=n; i++) {
      ConePoint(r,i*dphi,&v2,&v3,p+2);
      vn1=p[2];
      IncPoint3D(p[2],(*p1))@;
      ConePoint(r,i*dphi,&u2,&u3,p+3);
      vn2=p[3];
      IncPoint3D(p[3],(*p2));
      glVertex3d(p[2].x,p[2].y,p[2].z);
      glNormal3d(vn1.x,vn1.y,vn1.z);

      glVertex3d(p[3].x,p[3].y,p[3].z);
      glNormal3d(vn2.x,vn2.y,vn2.z);

    }
  glEnd();

}

@ @<Skip coninuation if the angle between the vectors is to large@>=
  if(Dot(&u1,&v1)<CosSkipCont) {
    MakeCone(p1,p2,r,n);
    MakeCone(p2,p3,r,n);
    return;
   } 

@* Read 3Scripts.
@<Read ThreeSript files@>=
@<Threescript keywords@>@;
@<Check line for a keyword@>@;
@<Scan ThreeScript file@>@;

@ @<Threescript keywords@>=
#define KEYWORDNO 8
#define BBOX      0
#define VPOINT    1
#define LSOURCE   2
#define POINT     3
#define LINE      4
#define POLY      5
#define AMBIENT   6
#define COLOR     7

char *Keyword[KEYWORDNO]={"boundingbox",
              "viewpoint",
              "lightsources",
              "point",
              "line",
              "polygon",
              "ambient",
           "color"};

@ @<Check line for a keyword@>=
int SearchKeyWord(char *cline)
{ int i;
  for(i=0; i<KEYWORDNO; i++)
    if(cline==strstr(cline,Keyword[i])){
      return i;
     }
  return -1;
}


@ @d MAX_STRING (256) @<Scan ThreeScript file@>=
void Scan3ScriptFile(char *fname)
{ char cline[MAX_STRING];
  FILE *infile;
  int key,i;
  int plBuffer[2048];
  double x,y,z,red,green,blue,sred,sblue,sgreen;
  double psize,thickness;
  if(!(infile=fopen(fname,"r"))) return;
  strcpy(cline,"");
  red= green=blue=0.0; /* white */
  sred=sgreen=sblue=1.0;
  psize=thickness=0.001;
  while(!feof(infile)) {
     if(!*cline)
        fgets(cline,MAX_STRING,infile);
     key=SearchKeyWord(cline);
     if(key<0) {
        strcpy(cline,"");
        continue;
      }
     switch(key) {
       case BBOX   : @<Read bounding box@>@;    break;
       case VPOINT : @<Read viewpoint@>@;       break;
       case LSOURCE: @<Read light sources@>@;   break;
       case POINT  : @<Read point@>@;break;
       case LINE   : @<Read line@>@; break;
       case POLY   : @<Read polygon@>@;break;
       case AMBIENT: @<Read ambient light@>@; break;
       case COLOR  : @<Read color@>@; break;
      }
    }
  fclose(infile);
}

@ @<Read polygon@>=
i=0;
fgets(cline,MAX_STRING,infile);
do {
  sscanf(cline,"%lg %lg %lg\n",&x,&y,&z);
  plBuffer[i]=AddToPointPool(x,y,z);
  i++;
  fgets(cline,MAX_STRING,infile);
 }
while(SearchKeyWord(cline)<0 && !feof(infile));
Polygons=PrependLinePolyList(Polygons,Polygon3D,plBuffer,i,sred,sgreen,sblue);

@ @<Read line@>=
i=0;
fgets(cline,MAX_STRING,infile);
do {
  sscanf(cline,"%lg %lg %lg\n",&x,&y,&z);
  plBuffer[i]=AddToPointPool(x,y,z);
  i++;
  fgets(cline,MAX_STRING,infile);
 }
while(SearchKeyWord(cline)<0 && !feof(infile));
Lines=PrependLinePolyList(Lines,Line3D,plBuffer,i,red,green,blue);

@ @<Read point@>=
fscanf(infile,"%lg %lg %lg\n",&x,&y,&z);
i=AddToPointPool(x,y,z);
Points=PrependPointList(Points,i,psize,red,green,blue);
strcpy(cline,"");

@ @<Read bounding box@>=
fscanf(infile,"%lg %lg %lg\n",&(BoundP1.x),&(BoundP1.y),&(BoundP1.z));
fscanf(infile,"%lg %lg %lg\n",&(BoundP2.x),&(BoundP2.y),&(BoundP2.z));
strcpy(cline,"");

@ @<Read viewpoint@>=
fscanf(infile,"%lg %lg %lg\n",
  &(ViewPoint.x),
  &(ViewPoint.y),
  &(ViewPoint.z));
strcpy(cline,"");

@ @<Read color@>=
fscanf(infile,"%lg %lg %lg\n",&red,&green,&blue);
strcpy(cline,"");

@ @<Read light sources@>=
fgets(cline,MAX_STRING,infile);
i=0;
do {
  sscanf(cline,"%lg %lg %lg",
     &(LightPosition[i].x),&(LightPosition[i].y),&(LightPosition[i].z));
  fgets(cline,MAX_STRING,infile);
  sscanf(cline,"%lg %lg %lg",
     &(LightColor[i].r),&(LightColor[i].g),&(LightColor[i].b));
  fgets(cline,MAX_STRING,infile);
  LightNo++;
  i++;
  if(i>=MAX_LIGHT) break;
 }
while (SearchKeyWord(cline)<0);

@ @<Read ambient light@>=
fgets(cline,MAX_STRING,infile);
sscanf(cline,"%lg %lg %lg",& AmbientColor.r,& AmbientColor.g,& AmbientColor.b);
strcpy(cline,"");

@* OpenGL and GLUtil stuff related with the display of the object.
@d ToDegree (180.0/3.1415926)
@d ToRadian (3.1415926/180.0)
@d MVNearPlane (1.0)
@d MVFarPlane  (10.0)
@d MVViewAngle (45.0)

@<Display the data structure with OpenGL@>=
  @<Make a cone along $p_1$ and $p_2$@>@;
  @<Make a continued cone along $p_1$ and $p_2$ to $p_3$@>@;
  @<Sphere to draw points@>@;
  @<OpenGL output of points@>@;
  @<OpenGL output of lines and polygons@>@;
  @<Outlines of polygons@>@;
  @<Scale to unit volume@>@;
  @<Define lights@>@;
  @<Create the object@>@;
  @<Setup view parameters@>@;
  @<Initialise the view parameters@>@;


@ @d MAX(a,b) (a>b ? a : b)
@<Initialise the view parameters@>=
void InitView(void)
{
  glEnable(GL_DEPTH_TEST); 
  CreateObject();
  RecalcView(1.0);
}

@ @<Setup view parameters@>=
void RecalcView(double fac)
{ double vr, vphi, vtheta,bound1,bound2;
  Point3D vp;

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(MVViewAngle,
     1.0,
     MVNearPlane,MVFarPlane);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  SphericalFromCartesian(&ViewPoint, &vr,  &vtheta, &vphi);
  bound1=Norm(&BoundP1);
  bound2=Norm(&BoundP2);
  vp.x=fac*vr*cos(vphi)*sin(vtheta);
  vp.y=fac*vr*sin(vphi)*sin(vtheta);
  vp.z=fac*vr*cos(vtheta);
  CenterOfScene.x=0.5*(BoundP1.x+BoundP2.x);
  CenterOfScene.y=0.5*(BoundP1.y+BoundP2.y);
  CenterOfScene.z=0.5*(BoundP1.z+BoundP2.z);
  gluLookAt(vp.x,vp.y,vp.z, /* eye point */
     0.0,0.0,0.0,
     0.0,0.0,1.0); /* up is the $x$-axes and we use a right handed system */
  DefineLightSources();

  ScalingToUnitCube(BoundP1,BoundP2);
  glTranslated(-CenterOfScene.x,-CenterOfScene.y,-CenterOfScene.z);

  glPushMatrix();  /* dummy push matrix, so that we can pop when the
                      view is recalculated */
}


@ When the |SmoothShading| and |WireFrame| variables are changed we
  must recalculate the list of polygones with averaged normals and/or
  as wires.

@<Create the object@>=
void CreateObject(void)
{

  if(PointPool) {
    glPushMatrix();

    if(ObjID<0)
      ObjID=glGenLists(1);
    else {
      glDeleteLists(ObjID,1);
      ObjID=glGenLists(1);
     }

    glNewList(ObjID, GL_COMPILE);
    if(Points)
      GLPrintPointList(Points);
    if(Lines)
      GLPrintLinePolyList(Lines);
    if(Polygons)
      GLPrintLinePolyList(Polygons);
    if(PolyOutline)
      GLPrintOutlinePolyList(Polygons,White);
    glEndList();
    glPopMatrix();
   }
}

@ This is the id-number of the compiled list object if it is greater than
zero otherwise there is nothing to draw.
@<Global values@>=
int ObjID=-1;




@ @<Scale to unit volume@>=
void ScalingToUnitCube(Point3D p1, Point3D p2)
{ Point3D scale={1.0,1.0,1.0};
  double is;

  ScaleOfScene.x=scale.x=0.5*(p2.x-p1.x);
  ScaleOfScene.y=scale.y=0.5*(p2.y-p1.y);
  ScaleOfScene.z=scale.z=0.5*(p2.z-p1.z);
  MaxScaleOfScene=MAX3(ScaleOfScene.x,ScaleOfScene.y,ScaleOfScene.z);
  if(MaxScaleOfScene!=0.0)
    is=1.0/MaxScaleOfScene;
  else
    is=1.0;
  if(scale.x==0.0) scale.x=1.0;
  if(scale.y==0.0) scale.y=1.0;
  if(scale.z==0.0) scale.z=1.0;
/*  glScaled(1.0/scale.x,1.0/scale.y,1.0/scale.z);  */
  glScaled(is,is,is);
}

@ @<Define lights@>=
void DefineLightSources(void)
{ int i;
  float flt[4]={0.0f,0.0f,0.0f,1.0f};
  float material_diff[4]={1.0f,1.0f,1.0f,1.0f};
  float material_spec[4]={0.5f,0.5f,0.5f,1.0f};
  float bmaterial_diff[4]={0.5f,0.5f,0.5f,1.0f};
  float bmaterial_spec[4]={0.0f,0.0f,0.0f,1.0f};

  @<Set up the lights@>@;
  glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,material_diff);
  glMaterialfv(GL_FRONT,GL_SPECULAR,material_spec);
  glMaterialfv(GL_BACK,GL_AMBIENT_AND_DIFFUSE,bmaterial_diff);
  glMaterialfv(GL_BACK,GL_SPECULAR,bmaterial_spec);
}

@ @<Set up the lights@>=
  glEnable(GL_LIGHTING);
  for(i=0; i<LightNo; i++) {
    glEnable(GL_LIGHT0+i);
    flt[0]=LightPosition[i].x; @+ flt[1]=LightPosition[i].y; @+ flt[2]=LightPosition[i].z;
    flt[3]=0.0;
    glLightfv(GL_LIGHT0+i,GL_POSITION,flt);
    flt[0]=LightColor[i].r; @+ flt[1]=LightColor[i].g; @+ flt[2]=LightColor[i].b;
    flt[3]=1.0;
    glLightfv(GL_LIGHT0+i,GL_SPECULAR,flt);
    flt[0]=LightColor[i].r; @+
    flt[1]=LightColor[i].g; @+
    flt[2]=LightColor[i].b;
    glLightfv(GL_LIGHT0+i,GL_DIFFUSE,flt);
    flt[0]=AmbientColor.r; @+ flt[1]=AmbientColor.g; @+ flt[2]=AmbientColor.b;
    glLightfv(GL_LIGHT0+i,GL_AMBIENT,flt);

   }

@ Setup the polygones that form a sphere at |cntr| with radius
  |r| with |nphi| azimutal segments and |ntheta| polar segments.
@d PointSphereSegmentsPhi 18
@d PointSphereSegmentsTheta 9
@<Sphere to draw points@>=
void PointSphere(Point3D *cnt, double r, int nphi, int ntheta,
         float colorRed, float colorGreen, float colorBlue)
{ int i,j,count;
  double phi,theta;
  double dphi,dtheta;
  register double cp,sp,ct,st;
  register double cp1,sp1,ct1,st1;
  Point3D p0,p;
  float mat[4]={1.0f,1.0f,1.0f,1.0f};


  dphi=TWO_PI/nphi;
  dtheta=PI/ntheta;
  mat[0]=colorRed; @+   mat[1]=colorGreen; @+  mat[1]=colorBlue;
  glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,mat);

  for(i=0,count=0; i<nphi; i++) {
    cp=cos(phi=i*dphi);
    sp=sin(phi);
    cp1=cos(phi=(i+1)*dphi);
    sp1=sin(phi);
    for(j=0; j<ntheta; j++) {
      ct=cos(theta=j*dtheta);
      st=sin(theta);
      ct1=cos(theta=(j+1)*dtheta);
      st1=sin(theta);
      if(count++ & 1) {
        p.x=cp*st; @+ p.y=sp*st; @+p.z=ct;
        p0=p;
        @<Draw left lower polygon@>@;
        @<Draw right upper polygon@>@;
       }
      else {
    p.x=cp*st1; @+ p.y=sp*st1; @+ p.z=ct1;
    p0=p;
    @<Draw left upper polygon@>@;
    @<Draw right lower polygon@>@;
      }
    }
   }
  mat[0]=1.f; @+   mat[1]=1.f; @+  mat[1]=1.0;
  glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,mat);
}

@ @<Draw left lower polygon@>=
      glBegin(GL_POLYGON);
      @<Set vertex and normal from |p| and |cnt|@>@;
      p.x=cp1*st; @+ p.y=sp1*st;
      @<Set vertex and normal from |p| and |cnt|@>@;
      p.x=cp1*st1; @+ p.y=sp1*st1; @+p.z=ct1;
      @<Set vertex and normal from |p| and |cnt|@>@;
      glEnd();

@ @<Draw right upper polygon@>=
      glBegin(GL_POLYGON);
      @<Set vertex and normal from |p| and |cnt|@>@;
      p.x=cp*st1; @+ p.y=sp*st1; @+ p.z=ct1;
      @<Set vertex and normal from |p| and |cnt|@>@;          
      p=p0;
      @<Set vertex and normal from |p| and |cnt|@>@;      
      glEnd();

@ @<Draw left upper polygon@>=
    glBegin(GL_POLYGON);
    @<Set vertex and normal from |p| and |cnt|@>@;
        p.x=cp*st; @+ p.y=sp*st; @+ p.z=ct;
    @<Set vertex and normal from |p| and |cnt|@>@;
        p.x=cp1*st; @+ p.y=sp1*st; @+ p.z=ct;
    @<Set vertex and normal from |p| and |cnt|@>@;
    glEnd();

@ @<Draw right lower polygon@>=
    glBegin(GL_POLYGON);
    @<Set vertex and normal from |p| and |cnt|@>@;
        p.x=cp1*st1; @+ p.y=sp1*st1; @+ p.z=ct1;
    @<Set vertex and normal from |p| and |cnt|@>@;
    p=p0;
    @<Set vertex and normal from |p| and |cnt|@>@;
    glEnd();


@ @<Set vertex and normal from |p| and |cnt|@>=  
      glVertex3d(r*p.x+cnt->x,r*p.y+cnt->y,r*p.z+cnt->z);
      glNormal3d(p.x,p.y,p.z);

@ @<OpenGL output of points@>=
void GLPrintPointList(PointList *lst)
{
  Point3D *p;

  if(0.0>=PointSphereSize) {
    glDisable(GL_LIGHTING);
    while(lst) {
      p=PointPool + lst->point;
      glBegin(GL_POINTS);
      glVertex3d(p->x,p->y,p->z);
      glEnd();
      lst=lst->next;
     }
   glEnable(GL_LIGHTING);
  }
 else {
   while(lst) {
     p=PointPool + lst->point;
     PointSphere(p,lst->size*PointSphereSize,
           PointSphereSegmentsPhi,
           PointSphereSegmentsTheta,
           lst->color.r,
           lst->color.g,
           lst->color.b
         );
     lst=lst->next;
   }
 }

}

@ @<OpenGL output of lines and polygons@>=
void GLPrintLinePolyList(LinePolyList *lst)
{ int i,thisI;
  int j1,j2,j3;
  Point3D p,normal;
  float mat[4]={1.0f,1.0f,1.0f,1.0f};
  while(lst) {
    switch(lst->what) {
      case Polygon3D :
        mat[0]=(float)lst->color.r;
        mat[1]=(float)lst->color.g;
        mat[2]=(float)lst->color.b;
        glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,mat);
        mat[0]=(float)lst->color.r;
        mat[1]=(float)lst->color.g;
        mat[2]=(float)lst->color.b;
        glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,mat);

        if(WireFrame)
          glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
        else
          glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
        glBegin(GL_POLYGON);
        for(i=0; i<lst->pno; i++) {
          p=PointPool[thisI=lst->points[i]];
          if(NormalPool[thisI] && SmoothShading)
             glNormal3d( NormalPool[thisI]->x,
                         NormalPool[thisI]->y,
                         NormalPool[thisI]->z);
          else {
            @<Cyclic index |j1|, |j2| and |j3|@>@;
            if(SurfaceNormal(&normal,PointPool+j1,PointPool+j2,PointPool+j3))
              glNormal3d(normal.x,normal.y,normal.z);
           }
         glVertex3d( p.x, p.y, p.z);
        }
      glEnd();
      break;
    case Line3D :
      if(0.0<LineTubeSize)
        @<Draw lines as tubes@>@;
      else
         @<Draw lines as lines@>@;
      break;
    }
    lst=lst->next;
   }
}

@ @<Draw lines as tubes@>=
{ if(2==lst->pno)
    MakeCone(PointPool+lst->points[0],
         PointPool+lst->points[1],
         LineTubeSize,TubeSegments);
  else {
    for(i=0; i<lst->pno-2; i++)
      MakeConeContinued(
        PointPool+lst->points[i],
         PointPool+lst->points[i+1],
         PointPool+lst->points[i+2],
        LineTubeSize,TubeSegments);
    i=lst->pno-2;
    if(lst->points[0]==lst->points[i+1]) { /* closed curve */
      MakeConeContinued(
        PointPool+lst->points[i],
        PointPool+lst->points[0],
        PointPool+lst->points[1],
        LineTubeSize,TubeSegments);
    }
   else
     MakeCone(PointPool+lst->points[i],
         PointPool+lst->points[i+1],
         LineTubeSize,TubeSegments);

  }
}

@ @<Draw lines as lines@>=
{
  glBegin(lst->pno > 2 ? GL_LINE_STRIP : GL_LINES);
  for(i=0; i<lst->pno; i++) {
    p=PointPool[lst->points[i]];
    glVertex3d(p.x,p.y,p.z);
   }
  glEnd();
}

@ @<Outlines of polygons@>=
void GLPrintOutlinePolyList(LinePolyList *lst,RGBColor c)
{ int i,thisI;
  int j1,j2,j3;
  Point3D p;

  glDisable(GL_LIGHTING);
  glLineWidth(2.0f);
  glColor3d(c.r,c.g,c.b);
  glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
  while(lst) {
    glBegin(GL_POLYGON);
    for(i=0; i<lst->pno; i++) {
      p=PointPool[thisI=lst->points[i]];
      glVertex3d( p.x, p.y, p.z);
     }
    glEnd();
    lst=lst->next;
   }
  glLineWidth(1.0f);
  glEnable(GL_LIGHTING);

}

@* POVray output.
@<Write POVray file@>=
  @<Write coordinates for POVray@>@;
  @<Check if normals exist for a polygon@>@;
  @<Write a single triangle for POVray@>@;
  @<Write the rotation of the trackball@>@;
  @<Make a cone for POVray along $p_1$ and $p_2$@>@;
  @<Make a continued cone for POVray along $p_1$ and $p_2$ to $p_3$@>@;
  @<Write the points into the POVray file@>@;
  @<Write lines@>@;
  @<Write triangle list@>@;
  @<Scale to original coordinates@>@;
  @<Write light source@>@;
  @<Write POVray header@>@;
  @<Save the data as POVray file@>@;

@ @<Scale to original coordinates@>=
void ScaleToWorld(Point3D *pw, Point3D *p)
{
  pw->x=CenterOfScene.x+MaxScaleOfScene* p->x;
  pw->y=CenterOfScene.y+MaxScaleOfScene* p->y;
  pw->z=CenterOfScene.z+MaxScaleOfScene* p->z;
}

@ @<Write coordinates for POVray@>=
void WritePOVVector(FILE *f,char *pre, Point3D *p,char *post)
{
  fprintf(f,"%s<%lg, %lg, %lg>%s",pre,p->x,p->y,p->z,post);
}

@ @<Write light source@>=
void WriteLights(FILE *f,Point3D *lightp, RGBColor *lcolor, int n,int shaddow)
{ Point3D ls;
  int i;

  for (i=0; i<n; i++){
    ScaleToWorld(&ls,lightp+i);
    WritePOVVector(f,"light_source {\n\t",&ls,"\n");
    WritePOVVector(f,"\tcolor rgb ",(Point3D *)(lcolor+i),"\n");
    if(!shaddow) fprintf(f,"\tshadowless\n");
    fprintf(f,"}\n\n");
   }
}

@ @<Write the rotation of the trackball@>=
void POVTrackballPosition(FILE *f)
{ char mstr[256];
  Point3D p1;

  p1.x=-CenterOfScene.x;
  p1.y=-CenterOfScene.y;
  p1.z=-CenterOfScene.z;
  WritePOVVector(f,"\ttranslate ",&p1,"\n");
  sprintf(mstr,"\trotate %lg*",ToDegree*TrackAngle);
  WritePOVVector(f,mstr,&TrackAxis,"\n");
  WritePOVVector(f,"\ttranslate ",&CenterOfScene,"\n");
}

@ @<Write the points into the POVray file@>=
void WritePOVPoints(FILE *f, PointList *lst)
{
  fprintf(f,"/* Begin Points */\n\n union {\n");
  while(lst) {
    fprintf(f,"\tsphere {\n");
    WritePOVVector(f,"\t",PointPool + lst->point," ");
    fprintf(f,"%lg \n\t}\n",PointSphereSize*lst->size);
    lst=lst->next;
   }
  if(TrackAngle!=0.0)
    POVTrackballPosition(f);
  fprintf(f,"}\n/* End of Points */\n");

}

@ @<Check if normals exist for a polygon@>=
int CheckNormals(int *points, int no)
{ int res,i;

  res=NULL != NormalPool[points[0]];
  for(i=1; i<no && res; i++)
    res = res && (NULL != NormalPool[points[i]]);
  return res;
}

@ @<Write triangle list@>=
void WritePOVTriangles(FILE *f, LinePolyList *lst, int smooth)
{ int i, thisI;
  Point3D *p;
  int isSmooth;

  fprintf(f,"/* Begin polygones */\n\n mesh {\n");
  while(lst) {
    isSmooth=smooth && CheckNormals(lst->points, lst->pno);
    if(lst->pno>=3) { /* allow only polygons with more than two points */ 
      fprintf(f,"%s {\n",isSmooth ? "smooth_triangle" : "triangle");
      for(i=0; i<lst->pno; i++) {
        p=PointPool+ (thisI=lst->points[i]);
        WritePOVVector(f,"\t",p,"");
        if(NormalPool[thisI] && isSmooth)
          WritePOVVector(f,", ", NormalPool[thisI],"");
        fprintf(f,"%s",i==lst->pno-1 ? "\n" : ",\n");
       }
      fprintf(f,"}\n");
     } 
    lst=lst->next;
  }
  if(TrackAngle!=0.0)
    POVTrackballPosition(f);
  fprintf(f,"}\n\n/* End polygones  */\n");
}

@ @<Write a single triangle for POVray@>=
void WritePOVSmoothTriangle(FILE *f,
       Point3D *p1, Point3D *p2, Point3D *p3,
       Point3D *n1, Point3D *n2, Point3D *n3)
{
  fprintf(f,"\tsmooth_triangle {\n");
  WritePOVVector(f,"\t",p1,", "); @+ WritePOVVector(f,"",n1,",\n");
  WritePOVVector(f,"\t",p2,", "); @+ WritePOVVector(f,"",n2,",\n");
  WritePOVVector(f,"\t",p3,", "); @+ WritePOVVector(f,"",n3,"\n");
  fprintf(f,"\t}\n");
}

@ @<Make a cone for POVray along $p_1$ and $p_2$@>=
void MakePOVCone(FILE *f,Point3D *p1, Point3D *p2, double r, int n)
{ Point3D v1,v2,v3,vn[4],p[4];
  double norm;
  double dphi;
  int i;

  v1.x=p2->x-p1->x;
  v1.y=p2->y-p1->y;
  v1.z=p2->z-p1->z;
  norm=Norm(&v1);
  v1.x/=norm; @+  v1.y/=norm; @+ v1.z/=norm;
  OrthogonalSet(&v1,&v2,&v3);

  v1.x=p2->x-p1->x;
  v1.y=p2->y-p1->y;
  v1.z=p2->z-p1->z;

  dphi=TWO_PI/n;
  ConePoint(r,0.0,&v2,&v3, p);
  vn[0]=vn[1]=*p;
  IncPoint3D(p[0],(*p1))@;
  AddPoint3D(p[1],p[0],v1)@;
  for(i=1; i<=n; i++) {
      ConePoint(r,i*dphi,&v2,&v3,p+2);
      vn[2]=p[2];
      IncPoint3D(p[2],(*p1))@;
      WritePOVSmoothTriangle(f,p,p+1,p+2,vn,vn+1,vn+2);
      AddPoint3D(p[3],p[2],v1)@;
      vn[3]=vn[2];
      WritePOVSmoothTriangle(f,p+2,p+1,p+3,vn+2,vn+1,vn+3); 
      p[0]=p[2]; @+ vn[0]=vn[2];
      p[1]=p[3]; @+ vn[1]=vn[3];
    }
}

@ @<Make a continued cone for POVray along $p_1$ and $p_2$ to $p_3$@>=
void MakePOVConeContinued(FILE *f,Point3D *p1, Point3D *p2, Point3D *p3,
               double r, int n)
{ Point3D v1,v2,v3,u1,u2,u3,vn[4],p[4];
  double norm;
  double dphi;
  int i;

  v1.x=p2->x-p1->x; @+ v1.y=p2->y-p1->y; @+ v1.z=p2->z-p1->z;
  norm=Norm(&v1);
  v1.x/=norm; @+  v1.y/=norm; @+ v1.z/=norm;
  OrthogonalSet(&v1,&v2,&v3);
  u1.x=p3->x-p2->x;@+  u1.y=p3->y-p2->y; @+  u1.z=p3->z-p2->z;
  norm=Norm(&u1);
  u1.x/=norm; @+  u1.y/=norm; @+ u1.z/=norm;
  @<Skip coninuation in POVray if the angle between the vectors is to large@>@;
  u2=v2; @+  u3=v3;
  GramSchmidt(&u1,&u2,&u3);
  dphi=TWO_PI/n;
  ConePoint(r,0.0,&v2,&v3, p);
  vn[0]=*p;
  IncPoint3D(p[0],(*p1))@;
  ConePoint(r,0.0,&u2,&u3,p+1);
  vn[1]=p[1];
  IncPoint3D(p[1],(*p2))
  for(i=1; i<=n; i++) {
    ConePoint(r,i*dphi,&v2,&v3,p+2);
    vn[2]=p[2];
    IncPoint3D(p[2],(*p1))@;
    WritePOVSmoothTriangle(f,p,p+1,p+2,vn,vn+1,vn+2);
    ConePoint(r,i*dphi,&u2,&u3,p+3);
    vn[3]=p[3];
    IncPoint3D(p[3],(*p2)) @;
    WritePOVSmoothTriangle(f,p+2,p+1,p+3,vn+2,vn+1,vn+3);
    p[0]=p[2]; @+ vn[0]=vn[2];
    p[1]=p[3]; @+ vn[1]=vn[3];

   }
}

@ @<Skip coninuation in POVray if the angle between the vectors is to large@>=
  if(Dot(&u1,&v1)<CosSkipCont) {
    MakePOVCone(f,p1,p2,r,n);
    MakePOVCone(f,p2,p3,r,n);
    return;
   }

@ @<Write lines@>=
void WritePOVLines(FILE *f, LinePolyList *lst)
{ int i,cnt;

  fprintf(f,"/* Begin lines */\n");
  if(0.0<LineTubeSize) {
    cnt=1;
    while(lst) {
      fprintf(f,"/* Line %d */\n",cnt);
      fprintf(f,"mesh {\n");
      @<Draw lines for POVray as tubes@>@;
      if(TrackAngle!=0.0)
        POVTrackballPosition(f);
      fprintf(f,"}\n/* End of Line %d*/\n",cnt++);
      lst=lst->next;
     }
   }
  else
    fprintf(f,"// Line output skiped due to zero LineTubeSize\n");
  fprintf(f,"/* End lines */");
}

@ @<Draw lines for POVray as tubes@>=
{ if(2==lst->pno)
    MakePOVCone(f,PointPool+lst->points[0],
         PointPool+lst->points[1],
         LineTubeSize,TubeSegments);
  else {
    for(i=0; i<lst->pno-2; i++)
      MakePOVConeContinued(f,
        PointPool+lst->points[i],
    PointPool+lst->points[i+1],
    PointPool+lst->points[i+2],
    LineTubeSize,TubeSegments);
    i=lst->pno-2;
    if(lst->points[0]==lst->points[i+1]) { /* closed curve */
      MakePOVConeContinued(f,
        PointPool+lst->points[i],
    PointPool+lst->points[0],
    PointPool+lst->points[1],
    LineTubeSize,TubeSegments);
    }
   else
     MakePOVCone(f,PointPool+lst->points[i],
         PointPool+lst->points[i+1],
         LineTubeSize,TubeSegments);
  }
}


@ @d Camera 2.33
 @<Write POVray header@>=
void WritePOVProlog(FILE *f, const char *povname)
{ Point3D vp,p;
  Point3D addLight[8]={{2,2,2},{-2,2,2},{-2,-2,2},{2,-2,2},
                       {2,2,-2},{-2,2,-2},{-2,-2,-2},{2,-2,-2}};
  RGBColor addLColor[8]={{0.25,0.25,0.25},
                         {0.25,0.25,0.25},{0.25,0.25,0.25},{0.25,0.25,0.25},
                        {0.1,0.1,0.1},{0.1,0.1,0.1},{0.1,0.1,0.1},{0.1,0.1,0.1}};
  fprintf(f,"//**********************************************\n"@|
        "//  File was created by " BANNER "\n// Version " VERSION "\n"@|
        "// (c) by Jens-Peer Kuska 1997\n"@|
        "//**********************************************\n");
  fprintf(f,"// POVray 3.0 input file: %s     \n",povname);
  if(*ThreeScriptName) {
    fprintf(f,"// 3Script source       : %s     \n",ThreeScriptName);
   }
  fprintf(f,"// Bounding box:\n"@|
        "// x: %lg  %lg\n// y: %lg %lg\n// z: %lg %lg \n//\n",@|
        BoundP1.x,BoundP2.x,BoundP1.y,BoundP2.y,BoundP1.z,BoundP2.z);
  fprintf(f,"// Unique Points: %d\n",PoolSize);
  fprintf(f,"// Line         : %ld\n",LengthLinePolyList(Lines));
  fprintf(f,"// Polygons     : %ld\n",LengthLinePolyList(Polygons));

  fprintf(f,"// end of header *******************************\n\n");
  fprintf(f,"#default {"@|
                    "texture {"@|
                     "pigment { color <1, 1, 1>}"@|
                       "finish { phong 0.8 ambient 0.4}"@|
                      "}}\n");
  if(AmbientColor.r>0.0 || AmbientColor.g>0.0 || AmbientColor.r>0.0)
    fprintf(f,"global_settings { ambient_light color rgb <%lg, %lg, %lg>}\n\n",
        AmbientColor.r,AmbientColor.g,AmbientColor.b);
  ScaleToWorld(&vp,&ViewPoint);
  p.x=p.z=0.0; @+ p.y=Norm(&vp);
  fprintf(f,"camera {\n");

  WritePOVVector(f,"\tlocation  ",&vp,"\n");
  WritePOVVector(f,"\tdirection ",&p,"\n");
  p.x=p.y=0.0; @+ p.z=Camera*MaxScaleOfScene;
  WritePOVVector(f,"\tup        ",&p,"\n");
  p.x=Camera*MaxScaleOfScene; @+ p.y=p.z=0.0; /* make a right hand coordiante system */
  WritePOVVector(f,"\tright     ",&p,"\n");
  WritePOVVector(f,"\tsky       ",&ZAxis,"\n");
  WritePOVVector(f,"\tlook_at   ",&CenterOfScene,"\n }\n\n");
  WriteLights(f,LightPosition,LightColor,LightNo,GL_FALSE);
  WriteLights(f,addLight,addLColor,8,GL_TRUE);
  fprintf(f,"/* Graphics data begin */\n");
}


@ @<Save the data as POVray file@>=
int WritePOVray(const char *povout, int smooth)
{ FILE *f;

  f=fopen(povout,"w");
  if(!f) return 1;
  WritePOVProlog(f,povout);
  if(Points && (0.0<PointSphereSize))
    WritePOVPoints(f,Points);
  if(Lines)
    WritePOVLines(f,Lines);

  if(Polygons)
    WritePOVTriangles(f,Polygons,smooth);
  fclose(f);
  return 0;
}

@* AutoCAD DXF files.

@<Stuff to write a DXF file@>=
@<Write a 3d face@>@;
@<Check the ordering of the tetragons@>@;
@<Write a triangle to the DXF file@>@;
@<Write a tetragon to the DXF file@>@;
@<Write a sphere at |p| into the DXF file@>@;
@<Write the points to the DXF file@>@;
@<Make a cone for the DXF file@>@;
@<Make a continued cone for the DXF output@>@;
@<Write the polygon list@>@;
@<Write the line list as covered lines to the DXF file@>@;
@<Write the DXF file@>@;


@ @<Write the DXF file@>=
int  WriteDXF(char *dxfname)
{ FILE *dxffile;

  dxffile=fopen(dxfname,"w");
  if(dxffile) {
    @<Write the DXF header@>@;
    @<Start the Entities section@>@;

    if(Polygons)
      WriteDXFPolys(dxffile,POLY_LAYER,1,Polygons);
    if(Lines && LineTubeSize>0.0)
      WriteDXFLines(dxffile,LINE_LAYER,2,Lines);
    if(Points && PointSphereSize>0.0)
      WriteDXFPoints(dxffile,POINT_LAYER,3,Points);

    @<Close the Entities section@>@;
    @<End of DXF file@>@;
    fclose(dxffile);
    return 0;
   }
  return 1;
}

@ The header section contains general information about the
  drawing, these are only relevant when creating
  specialised DXF files for AutoCAD. We will write a header comment
  and a empty header section. We will define three layers for
  polygons (layer 1), polygons created from lines (layer 2) and
  polygons created from points (layer 3).
@d POLY_LAYER  1
@d LINE_LAYER  2
@d POINT_LAYER 3

@<Write the DXF header@>=
  fprintf(dxffile,"999\n"
    "DXF created from mathview3d "VERSION
    "\n0\n"
    "SECTION\n2\n"
    "HEADER\n0\nENDSEC\n");
  /* Create 3 layers */
  fprintf(dxffile,"0\nSECTION\n2\nTABLES\n0\n"
                  "TABLE\n3\n"
                  "LAYER\n0\n"
                  "LAYER\n2\n1\n0\n"
                  "LAYER\n2\n2\n0\n"
                  "LAYER\n2\n3\n0\n"
                  "ENDTAB\n0\nENDSEC\n");
  /* Empty blocks section */
  fprintf(dxffile,"0\nSECTION\n2\nBLOCKS\n0\nENDSEC\n");

@ @<End of DXF file@>=fprintf(dxffile,"0\nEOF\n");

@ @<Start the Entities section@>=
fprintf(dxffile,"0\nSECTION\n2\nENTITIES\n");

@ @<Close the Entities section@>=
fprintf(dxffile,"0\nENDSEC\n");

@ @<Write a 3d face@>=
void WriteDXF3dFace(FILE *f, int layer,
        int color, int *points, int pno)
{ int i,k;

  fprintf(f,"0\n3DFACE\n8\n%d\n",layer);
  fprintf(f,"62\n%d\n",color);
  for(i=0; i<pno; i++) {
    k=1;
    fprintf(f,"%d\n%lg\n",i+10*(k++),PointPool[points[i]].x);
    fprintf(f,"%d\n%lg\n",i+10*(k++),PointPool[points[i]].y);
    fprintf(f,"%d\n%lg\n",i+10*(k++),PointPool[points[i]].z);
   }
}

@ DXF files need the repeated third point of a triangle.
@<Write a triangle to the DXF file@>=
void WriteDXFTriangle(FILE *f, int layer,
        int color, Point3D *p1, Point3D *p2, Point3D *p3)
{
  fprintf(f,"0\n3DFACE\n8\n%d\n",layer);
  fprintf(f,"62\n%d\n",color);
  fprintf(f,"%d\n%lg\n%d\n%lg\n%d\n%lg\n",
    10,p1->x,20,p1->y,30,p1->z);
  fprintf(f,"%d\n%lg\n%d\n%lg\n%d\n%lg\n",
    11,p2->x,21,p2->y,31,p2->z);
  fprintf(f,"%d\n%lg\n%d\n%lg\n%d\n%lg\n",
    12,p3->x,22,p3->y,32,p3->z);
  fprintf(f,"%d\n%lg\n%d\n%lg\n%d\n%lg\n",
    13,p1->x,23,p1->y,33,p1->z);
}

@ The polygons created by |MakeDXFCone| and |MakeDXFConeContinued| are
  not correctly oriented. For OpenGL and POVRay this doesn't
  matter because the surface normals are explicit given, for DXF files
  we must check the correct orientation.
@<Check the ordering of the tetragons@>=
int CheckTubeTetra(Point3D *p1,Point3D *p2, Point3D *p3, Point3D *p4,
                   Point3D *v1,Point3D *v2, Point3D *v3, Point3D *v4)
{ Point3D n;
  Point3D nn1,nn2;

  n.x=0.25*(v1->x+v2->x+v3->x+v4->x);
  n.y=0.25*(v1->y+v2->y+v3->y+v4->y);
  n.z=0.25*(v1->z+v2->z+v3->z+v4->z);
  SurfaceNormal(&nn1,p4,p1,p2);
  SurfaceNormal(&nn2,p2,p3,p4);
  nn1.x+=nn2.x; @+ nn1.y=nn2.y; @+ nn1.z=nn2.z;
  if(Dot(&nn1,&n)<0) return GL_FALSE;
  return GL_TRUE;
}

@ DXF files can store tetragons, the points must not lie in a plane
@<Write a tetragon to the DXF file@>=
void WriteDXFTetra(FILE *f, int layer,
        int color, Point3D *p1, Point3D *p2, Point3D *p3, Point3D *p4)
{
  fprintf(f,"0\n3DFACE\n8\n%d\n",layer);
  fprintf(f,"62\n%d\n",color);
  fprintf(f,"%d\n%lg\n%d\n%lg\n%d\n%lg\n",
    10,p1->x,20,p1->y,30,p1->z);
  fprintf(f,"%d\n%lg\n%d\n%lg\n%d\n%lg\n",
    11,p2->x,21,p2->y,31,p2->z);
  fprintf(f,"%d\n%lg\n%d\n%lg\n%d\n%lg\n",
    12,p3->x,22,p3->y,32,p3->z);
  fprintf(f,"%d\n%lg\n%d\n%lg\n%d\n%lg\n",
    13,p4->x,23,p4->y,33,p4->z);
}

@ @<Write the polygon list@>=
void WriteDXFPolys(FILE *f, int layer, int color, LinePolyList *lst)
{
  fprintf(f,"999\nPolygons\n");
  while(lst) {
    if(3==lst->pno)
      WriteDXFTriangle(f,layer,color,
        PointPool+lst->points[0],
        PointPool+lst->points[1],
        PointPool+lst->points[2]);
    else
      WriteDXF3dFace(f,layer,color,lst->points,lst->pno);
    lst=lst->next;
   }
}

@ @<Make a cone for the DXF file@>=
void MakeDXFCone(FILE *f,int layer, int color,
                 Point3D *p1, Point3D *p2, double r, int n)
{ Point3D v1,v2,v3,vn[4],p[4];
  double norm;
  double dphi;
  int i;

  v1.x=p2->x-p1->x;
  v1.y=p2->y-p1->y;
  v1.z=p2->z-p1->z;
  norm=Norm(&v1);
  v1.x/=norm; @+  v1.y/=norm; @+ v1.z/=norm;
  OrthogonalSet(&v1,&v2,&v3);

  v1.x=p2->x-p1->x;
  v1.y=p2->y-p1->y;
  v1.z=p2->z-p1->z;

  dphi=TWO_PI/n;
  ConePoint(r,0.0,&v2,&v3, p);
  vn[0]=vn[1]=*p;
  IncPoint3D(p[0],(*p1))@;
  AddPoint3D(p[1],p[0],v1)@;
  for(i=1; i<=n; i++) {
    ConePoint(r,i*dphi,&v2,&v3,p+2);
    vn[2]=p[2];
    IncPoint3D(p[2],(*p1))@;
    AddPoint3D(p[3],p[2],v1)@;
    vn[3]=vn[2];
    @<Write an oriented tetragon to the DXF file@>@;
    p[0]=p[2]; @+ vn[0]=vn[2];
    p[1]=p[3]; @+ vn[1]=vn[3];
   }
}

@ @<Write an oriented tetragon to the DXF file@>=
    if(CheckTubeTetra(p,p+1,p+3,p+2,vn,vn+1,vn+3,vn+2))
      WriteDXFTetra(f,layer,color,p,p+1,p+3,p+2);
    else
      WriteDXFTetra(f,layer,color,p+2,p+3,p+1,p);

@ @<Make a continued cone for the DXF output@>=
void MakeDXFConeContinued(FILE *f,int layer, int color,
               Point3D *p1, Point3D *p2, Point3D *p3,
               double r, int n)
{ Point3D v1,v2,v3,u1,u2,u3,vn[4],p[4];
  double norm;
  double dphi;
  int i;

  v1.x=p2->x-p1->x; @+ v1.y=p2->y-p1->y; @+ v1.z=p2->z-p1->z;
  norm=Norm(&v1);
  v1.x/=norm; @+  v1.y/=norm; @+ v1.z/=norm;
  OrthogonalSet(&v1,&v2,&v3);
  u1.x=p3->x-p2->x;@+  u1.y=p3->y-p2->y; @+  u1.z=p3->z-p2->z;
  norm=Norm(&u1);
  u1.x/=norm; @+  u1.y/=norm; @+ u1.z/=norm;
  @<Skip coninuation in DXF output@>@;
  u2=v2; @+  u3=v3;
  GramSchmidt(&u1,&u2,&u3);
  dphi=TWO_PI/n;
  ConePoint(r,0.0,&v2,&v3, p);
  vn[0]=*p;
  IncPoint3D(p[0],(*p1))@;
  ConePoint(r,0.0,&u2,&u3,p+1);
  vn[1]=p[1];
  IncPoint3D(p[1],(*p2))
  for(i=1; i<=n; i++) {
    ConePoint(r,i*dphi,&v2,&v3,p+2);
    vn[2]=p[2];
    IncPoint3D(p[2],(*p1))@;
    ConePoint(r,i*dphi,&u2,&u3,p+3);
    vn[3]=p[3];
    IncPoint3D(p[3],(*p2)) @;
    @<Write an oriented tetragon to the DXF file@>@;
    p[0]=p[2]; @+ vn[0]=vn[2];
    p[1]=p[3]; @+ vn[1]=vn[3];
   }
}

@ @<Skip coninuation in DXF output@>=
  if(Dot(&u1,&v1)<CosSkipCont) {
    MakeDXFCone(f,layer,color,p1,p2,r,n);
    MakeDXFCone(f,layer,color,p2,p3,r,n);
    return;
   }

@ @<Write the line list as covered lines to the DXF file@>=
void WriteDXFLines(FILE *f, int layer, int color, LinePolyList *lst)
{ int i;
  fprintf(f,"999\nCovered lines\n");
  while(lst) {
    @<Draw lines in the DXF file as tubes@>@;
    lst=lst->next;
   }
}

@ @<Draw lines in the DXF file as tubes@>=
{ if(2==lst->pno)
    MakeDXFCone(f,layer,color,PointPool+lst->points[0],
         PointPool+lst->points[1],
         LineTubeSize,TubeSegments);
  else {
    for(i=0; i<lst->pno-2; i++)
      MakeDXFConeContinued(f,layer,color,
        PointPool+lst->points[i],
    PointPool+lst->points[i+1],
    PointPool+lst->points[i+2],
    LineTubeSize,TubeSegments);
    i=lst->pno-2;
    if(lst->points[0]==lst->points[i+1]) { /* closed curve */
      MakeDXFConeContinued(f,layer,color,
        PointPool+lst->points[i],
    PointPool+lst->points[0],
    PointPool+lst->points[1],
    LineTubeSize,TubeSegments);
    }
   else
     MakeDXFCone(f,layer,color,PointPool+lst->points[i],
         PointPool+lst->points[i+1],
         LineTubeSize,TubeSegments);
  }
}

@ @<Write the points to the DXF file@>=
void WriteDXFPoints(FILE *dxffile,int layer,int color,PointList *lst)
{
  fprintf(dxffile,"999\nPoints\n");
  while(lst) {
    MakeDXFSphere(dxffile, layer, color,PointPool+(lst->point),
           lst->size*PointSphereSize,
           PointSphereSegmentsPhi,
           PointSphereSegmentsTheta);
    lst=lst->next;
   }

}

@ @<Write a sphere at |p| into the DXF file@>=
void MakeDXFSphere(FILE *dxffile,
       int layer, int color, Point3D *pnt,
       double r, int nphi, int nth)
{ double dth,dp;
  int i, j;
  double cp,cp1,st,st1,sp,sp1,ct,ct1,phi,th;
  Point3D p,p1,p2,p3;

  dp=TWO_PI/nphi;
  dth=PI/nth;
  for(i=0; i<nphi; i++) {
    cp=cos(phi=i*dp);
    sp=sin(phi);
    cp1=cos(phi=(i+1)*dp);
    sp1=sin(phi);
    for(j=0; j<nth; j++) {
      ct=r*cos(th=(j+1)*dth);
      st=r*sin(th);
      ct1=r*cos(th=j*dth);
      st1=r*sin(th);
      p.x=p1.x=p2.x=p3.x=pnt->x;
      p.y=p1.y=p2.y=p3.y=pnt->y;
      p.z=p1.z=p2.z=p3.z=pnt->z;
      p.x+=cp*st;    @+ p.y+=sp*st;    @+ p.z+=ct;
      p1.x+=cp1*st;  @+ p1.y+=sp1*st;  @+ p1.z+=ct;
      p2.x+=cp1*st1; @+ p2.y+=sp1*st1; @+ p2.z+=ct1;
      p3.x+=cp*st1;  @+ p3.y+=sp*st1;  @+ p3.z+=ct1;
      WriteDXFTetra(dxffile,layer,color,&p,&p1,&p2,&p3);
    }
   }
}

@* \ML-Interface.

@ Templates.
@<Templates for \.{mprep}@>=
@=:Begin:@>@|
@=:Function: mvGetPoolSize  @>@|
@=:Pattern: MViewPoolSize[] @>@|
@=:Arguments: {}            @>@|
@=:ArgumentTypes: {}        @>@|
@=:ReturnType: Integer      @>@|
@=:End:@>@|

@ @<Usage for functions in the Global context@>=
@=:Evaluate:@>@|
@=  MViewPoolSize::usage="MViewPoolSize returns the pool size of mathview3d."@>@|

@ @<Templates for \.{mprep}@>+=
@=:Begin: @>@|
@=:Function: mvPutGraphics3D @>@|
@=:Pattern: MVPutGraphics3D[l_List] @>@|
@=:Arguments:{l}                    @>@|
@=:ArgumentTypes: {Manual}          @>@|
@=:ReturnType: Manual               @>@|
@=:End:@>@|

@ @<Usage for functions in the Global context@>+=
@=:Evaluate:MVPutGraphics3D::usage="MVPutGraphics3D[lst] is internal used."@>@|

@ @<Templates for \.{mprep}@>+=
@=:Begin: @>@|
@=:Function: mvClearContents @>@|
@=:Pattern: MVClear[]       @>@|
@=:Arguments: {}            @>@|
@=:ArgumentTypes: {}        @>@|
@=:ReturnType: Manual       @>@|
@=:End:@>@|

@ @<Usage for functions in the Global context@>+=
@=:Evaluate:MVClear::usage="MVClear[] clears the mathview3d window."@>@|

@ @<Templates for \.{mprep}@>+=
@=:Begin: @>@|
@=:Function: mvReset @>@|
@=:Pattern: MVReset[]       @>@|
@=:Arguments: {}            @>@|
@=:ArgumentTypes: {}        @>@|
@=:ReturnType: Manual       @>@|
@=:End:@>@|

@ @<Usage for functions in the Global context@>+=
@=:Evaluate:MVReset::usage="MVReset[] reset the view parameter in the mathview3d window."@>@|

@ @<Templates for \.{mprep}@>+=
@=:Begin:@>@|
@=:Function: mvShowWindow  @>@|
@=:Pattern: MVShowWindow[] @>@|
@=:Arguments: {}           @>@|
@=:ArgumentTypes: {}       @>@|
@=:ReturnType: Manual      @>@|
@=:End:@>@|

@ @<Usage for functions in the Global context@>+=
@=:Evaluate:MVShowWindow::usage="MVShowWindow[] show the mathview3d window if it was hidden."@>@|

@ @<Templates for \.{mprep}@>+=
@=:Begin:@>@|
@=:Function: mvHideWindow  @>@|
@=:Pattern: MVHideWindow[] @>@|
@=:Arguments: {}           @>@|
@=:ArgumentTypes: {}       @>@|
@=:ReturnType: Manual      @>@|
@=:End:@>@|

@ @<Usage for functions in the Global context@>=
@=:Evaluate:MVHideWindow::usage="MVHideWindow[] hides the mathview3d window."@>@|

@ @<Templates for \.{mprep}@>+=
@=:Begin:@>@|
@=:Function: mvIconifyWindow  @>@|
@=:Pattern: MVIconifyWindow[] @>@|
@=:Arguments: {}           @>@|
@=:ArgumentTypes: {}       @>@|
@=:ReturnType: Manual      @>@|
@=:End:@>@|

@ @<Usage for functions in the Global context@>=
@=:Evaluate:MVIconifyWindow::usage="MVIconifyWindow[] iconifys the mathview3d window."@>@|

@ @<Templates for \.{mprep}@>+=
@=:Begin: @>@|
@=:Function: mvStopSpin @>@|
@=:Pattern: MVStopSpin[]       @>@|
@=:Arguments: {}            @>@|
@=:ArgumentTypes: {}        @>@|
@=:ReturnType: Manual       @>@|
@=:End:@>@|

@ @<Usage for functions in the Global context@>=
@:Evaluate:MVStopSpin::usage="MVStopSpin[] stops the spin animation of mathview3d."@>@|

@ @<Templates for \.{mprep}@>+=
@=:Begin: @>@|
@=:Function: mvZoom             @>@|
@=:Pattern: MVZoom[zoom_Real]   @>@|
@=:Arguments: {zoom}            @>@|
@=:ArgumentTypes: {Real}        @>@|
@=:ReturnType: Manual           @>@|
@=:End:@>@|

@ @<Usage for functions in the Global context@>+=
@=:Evaluate:MVZoom::usage="MVZoom[value] zooms in (value<1) or out (value>1)."@>@|

@ @<Templates for \.{mprep}@>+=
@=:Begin: @>@|
@=:Function: mvSpin                                                  @>@|
@=:Pattern: MVSpin[angle_Real,l_ /;                                  @>@|
@=            VectorQ[l,Real==Head[#] &] && 3==Length[l]]            @>@|
@=:Arguments: {angle,l}                                              @>@|
@=:ArgumentTypes:{Real,RealList}                                     @>@|
@=:ReturnType: Manual                                                @>@|
@=:End: @>

@ @<Templates for \.{mprep}@>+=
@=:Evaluate: MVSpin[angle_?NumericQ,{x_?NumericQ,y_NumericQ,z_NumericQ}]:= @>@|
@=             MVSpin[N[angle],N[{x,y,z}]] /. 0->0.0                    @>@|

@ @<Usage for functions in the Global context@>+=
@=:Evaluate:MVSpin::usage="MVSpin[angle,axis] starts spinning of the object@>@|
@=  in the mathview3d window around the axis {x,y,z} with angle increment angle."@>@|

@ @<Templates for \.{mprep}@>+=
@=:Begin: @>@|
@=:Function: mlWritePOVray                                  @>@|
@=:Pattern: mvWritePOVRay[fname_String,smooth_Integer,      @>@|
@=             ts_Real,tseg_Integer,psize_Real,tangle_Real] @>@|
@=:Arguments: {fname,smooth,ts,tseg,psize,tangle}           @>@|
@=:ArgumentTypes: {String,Integer,Real,Integer,Real,Real}   @>@|
@=:ReturnType: Manual                                       @>@|
@=:End:                                                     @>@|

@ @<Usage for functions in the Global context@>+=
@=:Evaluate:mvWritePOVRay::usage="mvWritePOVRay[] is used internaly by mathview3d."@>@|

@ @<Templates for \.{mprep}@>+=
@=:Begin:@>@|
@=:Function: mlWriteDXF                                      @>@|
@=:Pattern: mvWriteDXF[fname_String,                         @>@|
@=            ts_Real,tseg_Integer,psize_Real,tangle_Real]   @>@|
@=:Arguments: {fname,ts,tseg,psize,tangle}                   @>@|
@=:ArgumentTypes: {String,Real,Integer,Real,Real}            @>@|
@=:ReturnType: Manual                                        @>@|
@=:End:@>@|

@ @<Usage for functions in the Global context@>+=
@=:Evaluate:mvWriteDXF::usage="mvWriteDXF[] is used internaly by mathview3d."@>@|

@ @<Templates for \.{mprep}@>=
@<Usage for functions in the Global context@>@;

@=:Evaluate:@>@|
@=  Print["MathView3D installed, Version 1.1\\[NewLine]",@>@|
@=        "for ",$OperatingSystem," system ", $System,"\\[NewLine]",@>@|
@=        "\\[Copyright] by Jens-Peer Kuska 1997\\[NewLine]",@>@|
@=        "see: c't magazin f\\[UDoubleDot]r computer technik 9, 1997"];@>@|

@=:Evaluate:Unprotect["MathView3D`*"];@>@|

@=:Evaluate:@>@|
@=   BeginPackage["MathView3D`"]  @>@|

@<Usage for \Math\ functions and symbols@>@|

@=:Evaluate: Begin["`Private`"]@>@|
@=:Evaluate: SetAttributes[MVShow3D,ReadProtected]@>@|
@=:Evaluate: SetAttributes[MVWritePOVRay,ReadProtected]@>@|
@=:Evaluate: SetAttributes[MVWriteDXF,ReadProtected]@>@|
@=:Evaluate: SetAttributes[MVClose,ReadProtected]@>@|
@=:Evaluate: SetAttributes[MVCreatePalette,ReadProtected]@>@|

@<Additional \Math-definitions@>@;

@=:Evaluate: End[] (* Private *)@>@|

@=:Evaluate: EndPackage[]@>@|
@=:Evaluate: Protect["MathView3D`*"]@>@|

@ The function to close the link is a pure \Math-function.
@<Additional \Math-definitions@>=
@=:Evaluate:MVClose[]:= Uninstall /@@                                @>@|
@=            Select[Links[],StringMatchQ[First[#],"*mathview3d*"] &]@>@|

@ @<Usage for \Math\ functions and symbols@>=
@=:Evaluate: MVClose::usage="MVClose[] will uninstall the mathview3d program."@>@|

@ The \Math\ function \.{MVShow3D} will do the transfer
of the graphics data from \Math to \.{mathview3d}.
@<Additional \Math-definitions@>=
@=:Evaluate:@>@|
@=   Options[MVShow3D]:={MVLineTubeSize->0.01,               @>@|
@=             MVTubeSegments->12,                           @>@|
@=             MVTubeAngle->30,                              @>@|
@=               MVPointSphereSize-> 0.01,                     @>@|
@=               MVPolygonShading -> MVSmooth}                 @>@|

@=:Evaluate:@>@|
@=   mvShade={MVSmooth,MVFlat,MVWireFrame}                 @>@|

@=:Evaluate:@>@|
@=   mvShadeID[a_]:=Switch[a,                                        @>@|
@=            MVSmooth,       1,                               @>@|
@=            MVFlat,         2,                               @>@|
@=            MVWireFrame,    3,                               @>@|
@=            _,              1]                               @>@|


@ @<Additional \Math-definitions@>=
@=:Evaluate:@>@|
@=   MVShow3D[surf_SurfaceGraphics,opt___]:=                     @>@|
@=    MVShow3D[Graphics3D[surf],opt]                             @>@|
@=:Evaluate:@>@|
@=  MVShow3D::noint="MVTubeSegments must be an integer."@>@|
@=:Evaluate:@>@|
@=  MVShow3D::noreal="`1` must be a number."@>@|

@=:Evaluate:@>@|
@=   MVShow3D[gr_Graphics3D,opt___]:=                                @>@|
@=  Module[{vp,pltrange,light,lpos,lcol,a,col,                       @>@|
@=              ambient,ltube,psize,shade,args,tseg,tangle},         @>@|
@=    {pltrange,vp,light,ambient}=                                   @>@|
@=           FullOptions[gr,#] & /@@                                 @>@|
@=              {PlotRange,ViewPoint,LightSources,AmbientLight} /.   @>@|
@=         GrayLevel[x_] :> RGBColor[x,x,x];                 @>@|
@=    pltrange=Partition[Flatten[Transpose[pltrange]],3];        @>@|
@=    {lpos,lcol}=                                               @>@|
@=           Partition[Flatten[Transpose[light] /.                   @>@|
@=             RGBColor[args__] :> List[args],1],                    @>@|
@=             Length[light]];                                       @>@|
@=    ltube=MVLineTubeSize /. {opt} /.Options[MVShow3D];           @>@|
@=   If[!NumericQ[ltube],@>@|
@=     Message[MVShow3D::noreal,"MVLineTubeSize"];@>@|
@=     Return[$Failed];@>@|
@=    ];@>@|
@=   tangle=MVTubeAngle /. {opt} /. Options[MVShow3D];@>@|
@=   If[!NumericQ[tangle],@>@|
@=     Message[MVShow3D::noreal,"MVTubeAngle"];@>@|
@=     Return[$Failed];@>@|
@=    ];@>@|
@=    psize=MVPointSphereSize /. {opt} /.Options[MVShow3D];        @>@|
@=   If[!NumericQ[psize],@>@|
@=     Message[MVShow3D::noreal,"MVPointSphereSize"];@>@|
@=     Return[$Failed];@>@|
@=    ];@>@|
@=    shade=MVPolygonShading /. {opt}/.Options[MVShow3D];        @>@|
@=    shade=mvShadeID[shade];                                    @>@|
@=   tseg=MVTubeSegments /. {opt} /. Options[MVShow3D];          @>@|
@=    args=N[{                                                   @>@|
@=                pltrange,vp,                                   @>@|
@=                Flatten[lpos],Flatten[lcol],                   @>@|
@=                List @@@@ ambient,psize,                       @>@|
@=                ltube,                                         @>@|
@=                tangle} ] //. 0 ->0.0;                         @>@|
@=    AppendTo[args,shade];                                      @>@|
@=   AppendTo[args,tseg];                                        @>@|
@=    AppendTo[args,Flatten[N[{First[gr]}] //. 0 ->0.0] //.       @>@|
@=      { SurfaceColor[col_,__] :>  SurfaceColor[col],           @>@|
@=        Thickness[___] :> Sequence[],                          @>@|
@=        GrayLevel[a_]:> RGBColor[a,a,a],                       @>@|
@=        EdgeForm[___]:> Sequence[],                            @>@|
@=        Hue[__]:> Sequence[],                                  @>@|
@=        FaceForm[___]:> Sequence[]}];                          @>@|
@=     Global`MVPutGraphics3D[args]                              @>@|
@=   ]                                                           @>@|

@ @<Usage for \Math\ functions and symbols@>+=
@=:Evaluate:@>@|
@=   MVShow3D::usage=@>@|
@=     "MVShow3D[graph3d, opt...] will transfer a Graphics3D@>@|
@=      object to mathview3d."@>@|
@=:Evaluate:@>@|
@=   MVLineTubeSize::usage="MVLineTubeSize is an option for MVShow3D@>@|
@=   and MVWritePOVRay. The number given determines the absolute radius@>@|
@=   of the tubes arround lines."@>@|
@=:Evaluate:@>@|
@=   MVTubeSegments::usage="MVTubeSegments is an option for MVShow3D@>@|
@=   and MVWritePOVRay. The integer determines the number of segments@>@|
@=   for the tube approximation."@>@|
@=:Evaluate:@>@|
@=   MVTubeAngle::usage="MVTubeAngle is an option for MVShow3D@>@|
@=   and MVWritePOVRay. The angle (in degree) is used to determine if@>@|
@=   continued lines can smoothly continued with tubes."@>@|
@=:Evaluate:@>@|
@=   MVPointSphereSize::usage="MVPointSphereSize is an option for MVShow3D@>@|
@=   and MVWritePOVRay. The number gives the absolute size of spheres@>@|
@=   for point drawing."@>@|
@=:Evaluate:@>@|
@=   MVPolygonShading::usage="MVPolygonShading is an option for MVShow3D@>@|
@=   and MVWritePOVRay. The values MVSmooth (default), MVFlat and@>@|
@=   MVWireFrame are possible."@>@|
@=:Evaluate:@>@|
@=   MVSmooth::usage="MVSmooth is a value for MVPolygonShading."@>@|
@=:Evaluate:@>@|
@=   MVFlat::usage="MVFlat is a value for MVPolygonShading."@>@|
@=:Evaluate:@>@|
@=   MVWireFrame::usage="MVWireFrame is a value for MVPolygonShading."@>@|



@ The \Math\ function \.{MVWritePOVRay} will write a input
file for the Persistence Of Vision Raytracer. It takes the
same options as \.{MVShow3D} and alllow
different settings for the for the ray tracer file.
@ @<Additional \Math-definitions@>=
@=:Evaluate: Options[MVWritePOVRay]:=Options[MVShow3D]               @>@|
@=:Evaluate: MVWritePOVRay::noint="MVTubeSegments must be an integer." @>@|

@=:Evaluate: MVWritePOVRay::noreal="`1` must be a real number."@>@|

@ This tis the default usage. The data previously transfered
  to \.{mathview3d} are now saved in a output file \.{fname}
  for POVRay.

@<Additional \Math-definitions@>=
@=:Evaluate: MVWritePOVRay[fname_String,opts___Rule]:=          @>@|
@=              Module[{smth,ts,tseg,psize,tangle,filename},            @>@|
@=                 smth= MVPolygonShading /. {opts} /.              @>@|
@=                       Options[MVWritePOVRay];                @>@|
@=                 smth=mvShadeID[smth];                          @>@|
@=                 {ts,psize,tangle}=                                    @>@|
@=                 {MVLineTubeSize,MVPointSphereSize,MVTubeAngle}/. @>@|
@=                    {opts} /.   @>@|
@=                   Options[MVWritePOVRay] // N /. 0 ->0.0;    @>@|
@=                If[!NumericQ[ts],@>@|
@=                  Message[MVWritePOVRay::noreal,"MVLineTubeSize"];@>@|
@=                  Return[$Failed];@>@|
@=                 ];@>@|
@=                If[!NumericQ[psize],@>@|
@=                  Message[MVWritePOVRay::noreal,"MVPointSphereSize"];@>@|
@=                  Return[$Failed];@>@|
@=                 ];@>@|
@=               tseg=MVTubeSegments /. {opts} /.                @>@|
@=                   Options[MVWritePOVRay];                    @>@|
@=                  If[! IntegerQ[tseg] ,                         @>@|
@=                    Message[MVWritePOVRay::noint];              @>@|
@=                     Return[$Failed]                             @>@|
@=                    ];                                           @>@|
@=                If[!NumericQ[tangle],@>@|
@=                  Message[MVWritePOVRay::noreal,"MVTubeAngle"];@>@|
@=                  Return[$Failed];@>@|
@=                 ];@>@|
@=                filename=StringReplace[                       @>@|
@=                           fname,                             @>@|
@=                           "/"->$PathnameSeparator];          @>@|
@=                  Global`mvWritePOVRay[filename,smth,ts,tseg,psize,tangle]    @>@|
@=                 ]                                              @>@|

@ The second pattern will allow to send a 3d graphics to \.{mathview3d}
and to save it.
@<Additional \Math-definitions@>=
@=:Evaluate: MVWritePOVRay[fname_String,                        @>@|
@=                         gr_ /;                               @>@|
@=                         Head[gr]==Graphics3D ||              @>@|
@=                         Head[gr]==SurfaceGraphics,           @>@|
@=                         opts___Rule]:=                       @>@|
@=              (Global`MVClear[];MVShow3D[gr,opts];             @>@|
@=                MVWritePOVRay[fname,opts])                    @>@|

@ @<Usage for \Math\ functions and symbols@>+=
@=:Evaluate:@>@|
@=  MVWritePOVRay::usage="MVWritePOVRay[fname, opt..] writes the current@>@|
@=   object in the mathview3d window to the POVRay 3.0 input file fname.@>@|
@=   MVWritePOVRay[fname, graph3d, opt..] transfers the Graphics3D graph3d@>@|
@=   to mathview3d and savs it as a POVRay file."@>@|

@ @<Usage for \Math\ functions and symbols@>+=
@=:Evaluate:@>@|
@=  MVWriteDXF::usage="MVWriteDXF[fname,opt..] writes the current object@>@|
@=    in the mathview3d window to a DXF file fname.\n@>@|
@=    MVWriteDXF[fname,graph3d,opt..] transfer the Graphics3D to mathview3d @>@|
@=    and save a DXF file. Lines will only exported as as tubes, points as @>@|
@=    spheres."@>@|

@ The interface function to write AutoCAD DXF files.
@<Additional \Math-definitions@>+=
@=:Evaluate:Options[MVWriteDXF]:={MVLineTubeSize->0.01,@>@|
@=		MVTubeSegments->12,                              @>@|
@=		MVPointSphereSize->0.01,                         @>@|
@=		MVTubeAngle->30.}                                @>@|

@ @<Additional \Math-definitions@>+=
@=:Evaluate: MVWriteDXF::noreal="`1` must be a floating point number"@>@|

@ The standard pattern to write DXF files.
@<Additional \Math-definitions@>+=
@=:Evaluate:MVWriteDXF[fname_String,opt___?OptionQ]:=                @>@|
@=	  Module[{ts,tseg,psize,tangle,filename},                        @>@|
@=			ts=MVLineTubeSize /. {opt} /. Options[MVWriteDXF];           @>@|
@=			tseg=MVTubeSegments /. {opt} /. Options[MVWriteDXF];         @>@|
@=			psize=MVPointSphereSize /. {opt} /. Options[MVWriteDXF];     @>@|
@=			tangle=MVTubeAngle /. {opt} /. Options[MVWriteDXF];          @>@|
@=			{ts,psize,tangle}=N[{ts,psize,tangle}] /. 0-> 0.0;           @>@|
@=			If[!IntegerQ[tseg],                                          @>@|
@=				 Message[MVShow3D::noint,"MVTubeSegments"];                @>@|
@=				Return[$Failed];                                           @>@|
@=				];                                                         @>@|
@=			If[!NumericQ[ts],                                            @>@|
@=				  Message[MVWriteDXF::noreal,"MVLineTubeSize"];            @>@|
@=				Return[$Failed];                                           @>@|
@=				];                                                         @>@|
@=			If[!NumericQ[psize],                                         @>@|
@=				Message[MVWriteDXF::noreal,"MVPointSphereSize"];           @>@|
@=				Return[$Failed];                                           @>@|
@=				];                                                         @>@|
@=			If[!NumericQ[tangle],                                        @>@|
@=				Message[MVWriteDXF::noreal,"MVTubeAngle"];                 @>@|
@=				Return[$Failed];                                           @>@|
@=				];                                                         @>@|
@=			filename=StringReplace[fname,"/" -> $PathnameSeparator];     @>@|
@=			Global`mvWriteDXF[filename,ts,tseg,psize,tangle]             @>@|
@=			]                                                            @>@|

@ The pattern with a \.{Graphics3D}-object as second argument.
@<Additional \Math-definitions@>+=
@=:Evaluate:MVWriteDXF[fname_String,                                 @>@|
@=	          gr_ /;                                                 @>@|
@=	            Head[gr]==Graphics3D ||                              @>@|
@=	            Head[gr]==SurfaceGraphics,                           @>@|
@=	            opts___?OptionQ]:=                                   @>@|
@=	          (Global`MVClear[];                                     @>@|
@=	           MVShow3D[gr,opts];MVWriteDXF[fname,opts])             @>@|

@ Finaly we add a \Math 3.0 palette.
@ @<Additional \Math-definitions@>=
@=:Evaluate:@>@|
@=  mvPaletteHead =                              @>@|
@=   {                                           @>@|
@=    FrameBox[StyleBox[RowBox[{"Actions"}]]],   @>@|
@=    FrameBox[StyleBox[RowBox[{"Main"}   ]]],   @>@|
@=    FrameBox[StyleBox[RowBox[{"Spin"}   ]]],   @>@|
@=    FrameBox[StyleBox[RowBox[{"Window"} ]]]    @>@|
@=   };                                          @>@|
@=:Evaluate:@>@|
@=  mvShowButton = ButtonBox[                                       @>@|
@=                 StyleBox[@>@|
@=                   RowBox[{"MVShow3D","[","\\[Placeholder]","]"}]],@>@|
@=                 ButtonData :>                                    @>@|
@=                   BoxData[                                       @>@|
@=                     RowBox[{"MVShow3D","[","\[Placeholder]",",", @>@|
@=                     "MVLineTubeSize","->","0.01",",",       @>@|
@=                     "MVPolygonShading","->","MVSmooth",",", @>@|
@=                     "MVTubeSegments","->","12",",",         @>@|
@=                     "MVTubeAngle","->","30",",",            @>@|
@=                     "MVPointSphereSize","->","0.01","]"}]   @>@|
@=                  ]                                               @>@|
@=                ];                                                @>@|
@=:Evaluate:@>@|
@=  mvSPOVButton = ButtonBox[                                               @>@|
@=               StyleBox[@>@|
@=                 RowBox[{"Smooth"," ","MVWritePOVRay","[","\\[Placeholder]","]"}]], @>@|
@=               ButtonData :>                                              @>@|
@=                 BoxData[                                                 @>@|
@=                   RowBox[{"MVWritePOVRay", "[", "\\[Placeholder]",",",    @>@|
@=                           "PolygonShading", "->", "MVSmooth", "]"}]]];@>@|
@=:Evaluate:@>@|
@=  mvFPOVButton = ButtonBox[                                               @>@|
@=               StyleBox[@>@|
@=                 RowBox[{"Flat"," ","MVWritePOVray","[","\\[Placeholder]","]"}]],@>@|
@=               ButtonData :>                                              @>@|
@=                 BoxData[                                                 @>@|
@=                   RowBox[{"MVWritePOVRay", "[", "\\[Placeholder]", ",",   @>@|
@=                           "PolygonShading", "->", "MVFlat", "]"}]]];@>@|


@=:Evaluate:                                                @>@|
@=  mvClearButton = ButtonBox[                              @>@|
@=                StyleBox[                                 @>@|
@=                  RowBox[{"Clear", " ", "Window"}]],      @>@|
@=                  ButtonFunction :> Global`MVClear[],     @>@|
@=                  ButtonEvaluator -> Automatic];          @>@|

@=:Evaluate:                                                @>@|
@=  mvResetButton = ButtonBox[                              @>@|
@=                StyleBox[RowBox[{"Reset", " ", "view"}]], @>@|
@=                ButtonFunction :> Global`MVReset[],       @>@|
@=                ButtonEvaluator -> Automatic];            @>@|

@=:Evaluate:                                                @>@|
@=  mvZoomInButton = ButtonBox[                             @>@|
@=                 StyleBox[RowBox[{"Zoom", " ",  "in"}]],  @>@|
@=                 ButtonFunction :> Global`MVZoom[0.5],    @>@|
@=                 ButtonEvaluator -> Automatic];           @>@|

@=:Evaluate:                                                @>@|
@=  mvZoomOutButton = ButtonBox[                            @>@|
@=                  StyleBox[RowBox[{"Zoom", " ", "out"}]], @>@|
@=                  ButtonFunction :> Global`MVZoom[2.0],   @>@|
@=                  ButtonEvaluator -> Automatic];          @>@|

@=:Evaluate:                                                @>@|
@=  mvSpinXButton =  ButtonBox[                             @>@|
@=                 StyleBox[RowBox[{"X-axis"}]],            @>@|
@=                 ButtonFunction:> Global`MVSpin[N[Pi/10],{1.0,0.0,0.0}],@>@|
@=                 ButtonEvaluator->Automatic];                    @>@|

@=:Evaluate:                                                       @>@|
@=  mvSpinYButton=ButtonBox[                                       @>@|
@=              StyleBox[RowBox[{"Y-axis"}]],                      @>@|
@=              ButtonFunction:> Global`MVSpin[N[Pi/10],{0.0,1.0,0.0}],   @>@|
@=              ButtonEvaluator->Automatic];                       @>@|
@=:Evaluate:                                                       @>@|
@=  mvSpinZButton= ButtonBox[                                      @>@|
@=               StyleBox[RowBox[{"Z-axis"}]],                     @>@|
@=               ButtonFunction :>  Global`MVSpin[N[Pi/10], {0.0, 0.0, 1.0}],@>@|
@=               ButtonEvaluator -> Automatic];@>@|
@=:Evaluate:                                   @>@|
@=  mvSpinHaltButton =                         @>@|
@=    ButtonBox[                               @>@|
@=      StyleBox[RowBox[{"Stop"}]],            @>@|
@=      ButtonFunction :>  Global`MVStopSpin[],       @>@|
@=      ButtonEvaluator -> Automatic];         @>@|
@=:Evaluate:                                   @>@|
@=  mvHideWindowButton =                     @>@|
@=    ButtonBox[                             @>@|
@=      StyleBox[RowBox[{"Hide"}]],          @>@|
@=      ButtonFunction :>  Global`MVHideWindow[],   @>@|
@=      ButtonEvaluator -> Automatic];       @>@|
@=:Evaluate:                                 @>@|
@=  mvShowWindowButton =                     @>@|
@=    ButtonBox[                             @>@|
@=      StyleBox[RowBox[{"Show"}]],          @>@|
@=      ButtonFunction :>  Global`MVShowWindow[],   @>@|
@=      ButtonEvaluator -> Automatic];       @>@|
@=:Evaluate:                                 @>@|
@=  mvIconifyButton =                        @>@|
@=    ButtonBox[                             @>@|
@=      StyleBox[RowBox[{"Iconify"}]],       @>@|
@=      ButtonFunction :>  Global`MVIconifyWindow[],@>@|
@=      ButtonEvaluator -> Automatic];       @>@|
@=:Evaluate:                                 @>@|
@=  mvQuitButton =                           @>@|
@=    ButtonBox[                             @>@|
@=      StyleBox[RowBox[{"Quit"}]],          @>@|
@=      ButtonFunction :>  MVClose[],        @>@|
@=      ButtonEvaluator -> Automatic];       @>@|
@=:Evaluate:@>@|
@=  MVCreatePalette[] :=                                   @>@|
@=    Module[{pal},                                        @>@|
@=    pal = Cell[                                          @>@|
@=            BoxData[                                     @>@|
@=              GridBox[                                   @>@|
@=                {                                        @>@|
@=                  { " ", StyleBox[                       @>@|
@=                           RowBox[{"OpenGL",  "Viewer"}],@>@|        
@=                           FontFamily -> "Helvetica"     @>@|
@=                          ], " "," "},                   @>@|
@=                 mvPaletteHead,                          @>@|
@=                 {mvShowButton, mvClearButton, mvSpinXButton, mvShowWindowButton},@>@|
@=                 {mvSPOVButton, mvResetButton, mvSpinYButton, mvHideWindowButton},@>@|
@=                 {mvFPOVButton, mvZoomInButton, mvSpinZButton, mvIconifyButton},  @>@|
@=                 {FrameBox[                                                       @>@|
@=                    StyleBox[                                                     @>@|
@=                      RowBox[{"\\[Copyright]", " ", "1997",@>@|
@=                            " ", "by", " ",  "Jens-Peer", @>@|
@=                            " ", "Kuska"}                 @>@|
@=                       ]                                  @>@|
@=                     ]                                    @>@|
@=                   ],                                     @>@|
@=                   mvZoomOutButton, mvSpinHaltButton, mvQuitButton}},@>@|
@=                 RowSpacings -> 0,                                   @>@|
@=                 ColumnSpacings -> 0,                                @>@|
@=                 Global`GridDefaultElement :>    @>@|
@=                   ButtonBox[ "\[Placeholder]"]  @>@|
@=                ]                                                    @>@|
@=             ],   (* BoxData *)@>@|                                                
@=            "Input",           @>@|
@=            Active -> True,    @>@|
@=            CellFrame -> False @>@|
@=           ];  (* Cell *)      @>@|
@=        FrontEndExecute[       @>@|
@=          {FrontEnd`NotebookApply[FrontEnd`InputNotebook[], pal]}@>@|
@=         ]                                                       @>@|
@=       ]                                                         @>@|


@ @<Usage for \Math\ functions and symbols@>+=
@=:Evaluate:MVCreatePalette::usage=@>@|
@=  "MVCreatePalette[] will create a Mathematica 3.0 palette for mathview3d."@>@|

@ @<\ML-Functions@>=
@<Check the command line for \ML@>@;
@<Evaluate one message from \Math@>@;
@<Scan the graphics data@>@;
@<Get a \.{Graphics3D} object from \Math@>@;
@<Clear the display@>@;
@<Spin the graphics from \Math@>@;
@<Stop spin of the object@>@;
@<Send the pool size to \Math@>@;
@<Save a POVRay file from \Math@>@;
@<Save a DXF file from \Math@>@;
@<Reset the view parameter from \Math@>@;
@<Zoom the picture from \Math@>@;
@<Show the MathView3D window@>@;
@<Hide the MathView3D window@>@;
@<Iconify the MathView3D window@>@;

@ @<Send the pool size to \Math@>=
int mvGetPoolSize(void)
{
  return PoolSize;
}

@ @<Function prototypes@>=
void mvClearContents(void);

@ @<Clear the display@>=
void mvClearContents(void)
{ int i;
  if (PointPool) free(PointPool);
  PointPool=NULL;
  if(NormalPool)
    for(i=0; i<PoolSize; i++)
      if(NormalPool[i]) free(NormalPool[i]);
  if(NormalPool) free(NormalPool);
  NormalPool=NULL;
  MaxPoolSize=PoolSize=0;
  Points=RemovePointList(Points);
  Lines=RemoveLinePolyList(Lines);
  Polygons=RemoveLinePolyList(Polygons);
  glDeleteLists(ObjID,1);
  ObjID=-1;
  DoTracking=DoTrackSpin=GL_FALSE;
  ZoomMove.x=ZoomMove.y=ZoomMove.z=0.0;
  RadialZoom=1.0;
  TrackAngle=0.0;
  TrackAxis.x=TrackAxis.y=0.0;
  TrackAxis.z=1.0;
  ScreenXStart=ScreenX=0;
  ScreenYStart=ScreenY=0;

  InitView();
  CreateObject();
  @<Finish \Math-menu@>@;
}

@ @<Reset the view parameter from \Math@>=
void mvReset(void)
{
  @<Reset the global parameter@>@;
  @<Finish \Math-menu@>@;
}

@ @<Show the MathView3D window@>=
void mvShowWindow(void)
{
  glutShowWindow();
  @<Finish \Math-menu@>@;
}

@ @<Hide the MathView3D window@>=
void mvHideWindow(void)
{
  glutHideWindow();
  @<Finish \Math-menu@>@;
}

@ @<Iconify the MathView3D window@>=
void mvIconifyWindow(void)
{
  glutIconifyWindow();
  @<Finish \Math-menu@>@;
}

@ @<Zoom the picture from \Math@>=
void mvZoom(double zoom)
{ double vr,vphi,vtheta;

  RadialZoom*=zoom;
  @<Recalculate the zoom translation@>@;
  glutPostRedisplay();
  MLPutDouble(stdlink,RadialZoom);
}

@ @<Finish \Math-menu@>=
  glutPostRedisplay();
  MLPutSymbol(stdlink,"Null");

@ @<Spin the graphics from \Math@>=
void mvSpin(double angle,double *axis, long int n)
{ double atrack;

  if(3!=n) {
    MLPutSymbol(stdlink,"$Failed");
    return;
   }
  TrackAxis.x=axis[0];@+ TrackAxis.y=axis[1]; TrackAxis.z=axis[2];
  atrack=Norm(&TrackAxis);
  if(atrack!=0.0) {
    TrackAxis.x/=atrack;
    TrackAxis.y/=atrack;
    TrackAxis.z/=atrack;
   }
  else {
    MLPutSymbol(stdlink,"$Failed");
    return;
   }
  TrackAngle=angle;
  if(TrackAngle>0.0)
    DTrackAngle=MIN(TWO_PI/90.0,TrackAngle);
  else
    DTrackAngle=MAX(-TWO_PI/90.0,TrackAngle);
  DoTrackSpin=GL_TRUE;
  DoTracking=GL_TRUE;
  GetMouseTrack=GL_FALSE;
  MLPutSymbol(stdlink,"Null");
}

@ @<Stop spin of the object@>=
void mvStopSpin(void)
{
  @<Stop spining by parameter reset@>@;
  @<Finish \Math-menu@>@;
}

@ @<Function prototypes@>=
void mlWritePOVray(const char *, int, double, int, double,double);

@ @<Save a POVRay file from \Math@>=
void mlWritePOVray(const char *fname, int smooth,
                   double ts, int tseg, double pssize, double tangle)
{ int res;
  double tsSave,psSave,cosaSave;
  int tsegSave;
  char filename[512];
  int i,j,n;

  tsSave=LineTubeSize;
  LineTubeSize=ts;
  tsegSave=TubeSegments;
  TubeSegments=tseg;
  psSave=PointSphereSize;
  PointSphereSize=pssize;
  cosaSave=CosSkipCont;
  CosSkipCont=cos(ToRadian*(fabs(tangle)<MaxLSAngle ? tangle : MaxLSAngle));
  /* \.{Flat} or \.{WireFrame} given - no smooth shading */
  smooth= smooth>1 ? GL_FALSE : GL_TRUE;
  @<Remove double backslashs from file |fname|@>@;
  res=WritePOVray(filename,smooth);
  LineTubeSize=tsSave;
  TubeSegments=tsegSave;
  PointSphereSize=psSave;
  CosSkipCont=cosaSave;
  if(res)
    MLPutSymbol(stdlink,"$Failed");
  else
    MLPutString(stdlink,fname);
}

@ @<Remove double backslashs from file |fname|@>=
  n=strlen(fname);
  for(j=i=0; i<n; i++)
    if(fname[i]!='\\')
      filename[j++]=fname[i];
    else if(fname[++i]=='\\')
      filename[j++]='\\';
  filename[j++]=fname[n]; @+ filename[j+1]='\0';

@ @<Save a DXF file from \Math@>=
void mlWriteDXF(const char *fname,
                   double ts, int tseg, double pssize, double tangle)
{ int res;
  double tsSave,psSave,cosaSave;
  int tsegSave;
  char filename[512];
  int i,j,n;

  tsSave=LineTubeSize;
  LineTubeSize=ts;
  tsegSave=TubeSegments;
  TubeSegments=tseg;
  psSave=PointSphereSize;
  PointSphereSize=pssize;
  cosaSave=CosSkipCont;
  CosSkipCont=cos(ToRadian*(fabs(tangle)<MaxLSAngle ? tangle : MaxLSAngle));
  @<Remove double backslashs from file |fname|@>@;
  res=WriteDXF(filename);
  LineTubeSize=tsSave;
  TubeSegments=tsegSave;
  PointSphereSize=psSave;
  CosSkipCont=cosaSave;
  if(res)
    MLPutSymbol(stdlink,"$Failed");
  else
    MLPutString(stdlink,fname);
  MLEndPacket(stdlink);  
}

@ @d MLPutList(n) MLPutFunction(stdlink,"List",n)
  @<Get a \.{Graphics3D} object from \Math@>=
void mvPutGraphics3D(void)
{ long int arglen,n;
  int shade;
  double *vec;
  const char *head;
  long int count;

  if(MLCheckFunction(stdlink,"List",&arglen)) {
    @<Get the plot range@>@;
    @<Get the view point@>@;
    @<Get the light sources@>@;
    @<Get the ambient light@>@;
    MLGetDouble(stdlink,&PointSphereSize);
    MLGetDouble(stdlink,&LineTubeSize);
    MLGetDouble(stdlink,&LineSegmentAngle);
    @<Calculate the $\cos$ of |LineSegmentAngle|@>@;
    MLGetInteger(stdlink,&shade);
    MLGetInteger(stdlink,&TubeSegments);
    switch(shade) {
      case 1 :
        SmoothShading=GL_TRUE;
        WireFrame=GL_FALSE;
        break;
      case 2 :
        SmoothShading=GL_FALSE;
        WireFrame=GL_FALSE;
        break;
      case 3 :
        SmoothShading=GL_FALSE;
        WireFrame=GL_TRUE;
    break;
     }
    MLCheckFunction(stdlink,head,&count);
    GetGraphics3D(stdlink,head,count);
   }
  else {
    MLPutSymbol(stdlink,"$Failed");
    return;
   }
  MLPutFunction(stdlink,"List",3L);
    @<Respond plot range@>@;
    @<Respond view point@>@;
    @<Respond ambient light@>@;
  if (Polygons) AveragedNormals(Polygons);
  InitView();
  MakeNewModel=GL_TRUE;
  glutPostRedisplay();
}

@ @<Respond plot range@>=
    MLPutFunction(stdlink,"Rule",2L);
      MLPutSymbol(stdlink,"PlotRange");
      MLPutList(3L);
        MLPutList(2L);
          MLPutReal(stdlink,BoundP1.x);
          MLPutReal(stdlink,BoundP2.x);
        MLPutList(2L);
          MLPutReal(stdlink,BoundP1.y);
          MLPutReal(stdlink,BoundP2.y);
        MLPutList(2L);
          MLPutReal(stdlink,BoundP1.z);
          MLPutReal(stdlink,BoundP2.z);

@ @<Respond view point@>=
    MLPutFunction(stdlink,"Rule",2L);
      MLPutSymbol(stdlink,"ViewPoint");
      MLPutRealList(stdlink,& ViewPoint.x,3L);
                   
@ @<Respond ambient light@>=
    MLPutFunction(stdlink,"Rule",2L);
      MLPutSymbol(stdlink,"AmbientLight");
      MLPutRealList(stdlink,& AmbientColor.r,3L);

@ @<Get the plot range@>=
if(!MLCheckFunction(stdlink,"List",&n)) {
  MLPutSymbol(stdlink,"$Failed");
  return;
 } 
else {
  MLGetRealList(stdlink,& vec,&n);
  memcpy((void *) & BoundP1, vec, n*sizeof(double));
  MLDisownRealList(stdlink,vec,n);
  MLGetRealList(stdlink,& vec,&n);
  memcpy((void *) & BoundP2, vec, n*sizeof(double));
  MLDisownRealList(stdlink,vec,n);
 }

@ @<Get the view point@>=
MLGetRealList(stdlink,& vec,&n);
memcpy((void *) & ViewPoint, vec, n*sizeof(double));
MLDisownRealList(stdlink,vec,n);

@ @<Get the light sources@>=
MLGetRealList(stdlink, & vec, &n);
LightNo= n/3;
memcpy((void *) LightPosition, vec,n*sizeof(double));
MLDisownRealList(stdlink,vec,n);
MLGetRealList(stdlink, &vec, &n);
memcpy((void *) LightColor, vec,n*sizeof(double));
MLDisownRealList(stdlink,vec,n);

@ @<Get the ambient light@>=
  MLGetRealList(stdlink,& vec,&n);
  memcpy((void *) & AmbientColor, vec, n*sizeof(double));
  MLDisownRealList(stdlink,vec,n);
      
@ @<Scan the graphics data@>=
void GetGraphics3D(MLINK link,const char *head,long int count)
{ long ecount,n,coord,coord1,nlen,i;
  int k,j,plBuffer[2048];
  double *v, vec[3],vec1[3];
  int cubep[8];
  const char *fhead;
  double red,green,blue,sred,sgreen,sblue,dummy;
  double psize,thickness;

  fhead=head;
  ecount=count;
  n=ecount;
  red=green=blue=sred=sgreen=sblue=1.0;
  psize=thickness=0.001;
  for(i=0; i<ecount; i++) {
    @<Wait until new data passed from \Math@>@;
    MLGetFunction(link,&fhead,&n);
    if(0==strcmp(fhead,"Polygon")) {
      @<Read a polygon from \ML@>@;
     }
    else if(0==strcmp(fhead,"Line")) {
      @<Read a line from \ML@>@;
     }
    else if(0==strcmp(fhead,"Point")) {
      @<Read a point from \ML@>@;
     }
    else if(0==strcmp(fhead,"RGBColor")) {
      @<Read the rgb-color from \ML@>@;
     }
    else if(0==strcmp(fhead,"SurfaceColor")) {
      @<Read the surface color from \ML@>@;
     }
    else if(0==strcmp(fhead,"PointSize")) {
      MLGetReal(link,&psize);
     }
    else if(0==strcmp(fhead,"Cuboid")) {
     @<Read \Math\ \.{Cuboid} object@>@;
     }
    else if(0==strcmp(fhead,"EdgeForm")) {
      if(n)
	      MLGetReal(link,&dummy);
     }
    else if (0==strcmp(fhead,"Thickness")) {
      MLGetReal(link,&dummy);
      }
    else if(0==strcmp(fhead,"List")) {
      GetGraphics3D(link,fhead,n);
     } 
    else {
      MLDisownSymbol(link,fhead);
      MLGetNext(link);
      continue;
     }
    MLDisownSymbol(link,fhead);
   }

}

@ When a large amount of data is passed to \.{mathview3d} we must
wait until new data written to the link. Since we want avoid
nonportable acess to the event loop of the main window we suspend
the reading from the link.
@<Wait until new data passed from \Math@>=
while(!MLReady(link)) {
#ifdef __WIN32__
  Sleep(100);
#else
  usleep(100);
#endif
}

@ @<Read a polygon from \ML@>=
MLCheckFunction(link,"List",&n);
for(k=j=0; j<n; j++) {
  @<Wait until new data passed from \Math@>@;
  MLGetRealList(link,&v,&coord);
  if(3==coord) {
    plBuffer[k++]=AddToPointPool(v[0],v[1],v[2]);
   }
  MLDisownRealList(link,v,coord);
 }
Polygons=PrependLinePolyList(Polygons,Polygon3D,plBuffer,k,sred,sgreen,sblue);

@ @<Read a line from \ML@>=
MLCheckFunction(link,"List",&n);
for(k=j=0; j<n; j++) {
  @<Wait until new data passed from \Math@>@;
  MLGetRealList(link,&v,&coord);
  if(3==coord) {
    plBuffer[k++]=AddToPointPool(v[0],v[1],v[2]);
   }
  MLDisownRealList(link,v,coord);
 }
Lines=PrependLinePolyList(Lines,Line3D,plBuffer,k,red,green,blue);

@ @<Read a point from \ML@>=
MLGetRealList(link,&v,&coord);
if(3==coord) {
  k=AddToPointPool(v[0],v[1],v[2]);
  Points=PrependPointList(Points,k,psize,red,green,blue);
 }
MLDisownRealList(link,v,coord);

@ @<Read \Math\ \.{Cuboid} object@>=
MLGetRealList(link,&v,&coord);
vec[0]=v[0]; @+ vec[1]=v[1]; @+ vec[2]=v[2];
MLDisownRealList(link,v,coord);
if(2==n) { /* get the optional second corner of the cuboid */
  MLGetRealList(link,&v,&coord1);
  vec1[0]=v[0]; @+ vec1[1]=v[1]; @+ vec1[2]=v[2];
  MLDisownRealList(link,v,coord1);
 }
else {
  vec1[0]=vec[0]+1.0; @+  vec1[1]=vec[1]+1.0; @+  vec1[2]=vec[2]+1.0;
  coord1=3;
 }
if(3==coord && 3==coord1) {
  plBuffer[3]=cubep[0]=AddToPointPool( vec[0],  vec[1],  vec[2]);
  plBuffer[2]=cubep[1]=AddToPointPool(vec1[0],  vec[1],  vec[2]);
  plBuffer[1]=cubep[2]=AddToPointPool(vec1[0], vec1[1],  vec[2]);
  plBuffer[0]=cubep[3]=AddToPointPool( vec[0], vec1[1],  vec[2]);
  plBuffer[4]=cubep[4]=AddToPointPool( vec[0],  vec[1], vec1[2]);
  plBuffer[5]=cubep[5]=AddToPointPool(vec1[0],  vec[1], vec1[2]);
  plBuffer[6]=cubep[6]=AddToPointPool(vec1[0], vec1[1], vec1[2]);
  plBuffer[7]=cubep[7]=AddToPointPool( vec[0], vec1[1], vec1[2]);
  Polygons=PrependLinePolyList(Polygons,Polygon3D,plBuffer,4,sred,sgreen,sblue);
  Polygons=PrependLinePolyList(Polygons,Polygon3D,plBuffer+4,4,sred,sgreen,sblue);
  plBuffer[0]=cubep[0]; @+  plBuffer[1]=cubep[1];
  plBuffer[2]=cubep[5]; @+  plBuffer[3]=cubep[4];
  Polygons=PrependLinePolyList(Polygons,Polygon3D,plBuffer,4,sred,sgreen,sblue);
  plBuffer[0]=cubep[1]; @+  plBuffer[1]=cubep[2];
  plBuffer[2]=cubep[6]; @+  plBuffer[3]=cubep[5];
  Polygons=PrependLinePolyList(Polygons,Polygon3D,plBuffer,4,sred,sgreen,sblue);
  plBuffer[0]=cubep[2]; @+  plBuffer[1]=cubep[3];
  plBuffer[2]=cubep[7]; @+  plBuffer[3]=cubep[6];
  Polygons=PrependLinePolyList(Polygons,Polygon3D,plBuffer,4,sred,sgreen,sblue);
  plBuffer[0]=cubep[3]; @+  plBuffer[1]=cubep[0];
  plBuffer[2]=cubep[4]; @+  plBuffer[3]=cubep[7];
  Polygons=PrependLinePolyList(Polygons,Polygon3D,plBuffer,4,sred,sgreen,sblue);

}


@ @<Read the rgb-color from \ML@>=
MLGetReal(link,&red);
MLGetReal(link,&green);
MLGetReal(link,&blue);

@ @<Read the surface color from \ML@>=
if(MLCheckFunction(link,"RGBColor",&n)) {
  MLGetReal(link,&sred);
  MLGetReal(link,&sgreen);
  MLGetReal(link,&sblue);
 }


@ We can't call MLMain because this will initiate a loop waiting
on \Math requests. This loop alone  causes that no events from the
window system are evaluated so we split |MLMain| into three parts.
@ @<Global values@>=
       MLINK LinkToMath;
       int MathLinkMode=GL_FALSE;
       int MathLinkActive=GL_FALSE;

@ @<Initialise the \ML-connection@>=
if (MathLinkMode) {
  if(!stdenv)
    stdenv = MLInitialize( (MLParametersPointer)0);
  if(stdenv == (MLEnvironment)0)
    return !MLDone;
  if( !stdhandler)
    stdhandler = MLCreateMessageHandler( stdenv, MLDefaultHandler, 0);
    LinkToMath = MLOpenArgv( stdenv, argv, argv + argc, &err);
    if( LinkToMath == (MLINK)0){
      MLAlert( stdenv, MLErrorString( stdenv, err));
      @<Close \ML-connection@>@;
     }
    if( stdhandler) MLSetMessageHandler( LinkToMath, stdhandler);
    if(!MLInstall( LinkToMath)) {
      @<Close \ML-connection@>@;
     }
    MathLinkActive=GL_TRUE;
}


@ @<Close \ML-connection@>=
    if(LinkToMath)
      MLClose(LinkToMath);
    MLDeinitialize( stdenv);
    stdenv = (MLEnvironment)0;
    exit(!MLDone);


@ @<Function prototypes@>=
int  ProcessMathLink(MLINK *mlp);

@ @<Evaluate one message from \Math@>=
int  ProcessMathLink(MLINK *mlp)
{ int pkt;

  if(MLDone && MLError(*mlp)) {
    return 0;
  }
  while(MLReady(*mlp) && !MLDone) {
    pkt=MLNextPacket(*mlp);
    MLAbort =0;
    if(RESUMEPKT==pkt) {
      if(!refuse_to_be_a_frontend(*mlp)) return 0; 
      @<Close \ML-connection@>@;
     }
    if(pkt!=CALLPKT || !MLDoCallPacket(*mlp)) return 0;
    MLFlush(*mlp);
    MLAbort =0;
  }
 return 1;
}


@ @<Check the command line for \ML@>=
int IsMathLinkCall(int argc, char *argv[])
{ int i;

  for(i=0; i<argc; i++) {
    if(strcmp(argv[i],"-mathlink")==0) return GL_TRUE;
    if(strcmp(argv[i],"-linkname")==0) return GL_TRUE;
    if(strcmp(argv[i],"-linkmode")==0) return GL_TRUE;
    if(strcmp(argv[i],"-linkcreate")==0) return GL_TRUE;
    if(strcmp(argv[i],"-linkprotocol")==0) return GL_TRUE;
   }
  return GL_FALSE;
}

@ @<Header@>=
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <GL/glut.h>
#include <mathlink.h>
#include <cmdln.h>
#if defined(__WIN32__) && defined(__BORLANDC__)
#  include <float.h>
#else
# include <unistd.h>
#endif
