/**********************************************************************
File : cmdln.c
Date : /01/17/94
ANSI C
J.-P. Kuska
scanning the command line of a C program in a convinent way
***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cmdln.h"
/**********************************************************************/
#define NEW(l)  (l *)malloc(sizeof(l))

#define YESNO_STR(bool)   ((bool) ? "Yes" : "No")

#if !defined(TRUE)

#define TRUE  1
#define FALSE 0

#endif

CmdArgLst *DeclareCmdArg(char *cmd_switch, 
                         int data_type,
                         void *data,
                         char *help)
/*************************************************************
This function produces one entry in the list of
commandline switches. 

cmd_switch     is a string (even if the switch only contain one 
               character) indicating the command line argument.
data_type      can be one of the following values
               CMD_SWITCH, CMD_INTEGER, CMD_DOUBLE, CMD_STRING
               the form of the command line argument is for 
               CMD_SWITCH   +<switch> for on and -<switch> for
               off, in all other cases the form is
               -<switch> <argument>.
data           a pointer to the variable needed to store the
               argument of the switch or TRUE or FALSE
               for the data have to point to
               CMD_SWITCH, CMD_INTEGER  *data  <== & int
               CMD_DOUBLE               *data  <== & double
               CMD_STRING               *data  <== & char
help           is a string that contains a short explanation of the
               meaning of the switch.

**************************************************************/
  { CmdArgLst  *lst;
     
     if ( lst = NEW(CmdArgLst) ) {
       lst->next=NULL;
       lst->CmdLetter=(char *)malloc((strlen(cmd_switch)+1)*sizeof(char));
       if(lst->CmdLetter)
         strcpy(lst->CmdLetter,cmd_switch);
       lst->What=data_type;
       lst->ArgumentGiven=FALSE;
       lst->DataPtr=data;
       lst->HelpTxt=(char *)malloc((strlen(help)+1)*sizeof(char));
       if (lst->HelpTxt)
         strcpy(lst->HelpTxt,help);
      }
     return lst;
   }
    

CmdArgLst *AppendCmdArg(CmdArgLst *new_cmd, CmdArgLst *lst)
/************************************************************
Add new_cmd to the list lst of possible command line switches.
*************************************************************/
  { CmdArgLst *head;

    head=lst;
    while(lst->next) lst=lst->next;
    lst->next=new_cmd;
    return head;
  }


void HelpCmdArg(char *banner,
                CmdArgLst *lst)
/*************************************************************
Print the banner and the help information in the list of
command line switches. If the DEBUG directive is given to
the preprocessor than a detailed information about the paramter
scanned ist also printed.
**************************************************************/
  {
     printf("%s \n""usage:\n\n",banner);
     while (lst) {
       switch (lst->What) {
         case CMD_SWITCH  : printf("+%s/-%s \t\t   %s",lst->CmdLetter,lst->CmdLetter,lst->HelpTxt);
#ifdef DEBUG
                            printf(" Info : %s  given %s ",*((int *) lst->DataPtr) ?  "On"  : "Off",
                                                          YESNO_STR(lst->ArgumentGiven));
#endif
                            break;
         case CMD_INTEGER : printf("-%-5s  <integer>  %s",lst->CmdLetter,lst->HelpTxt);
#ifdef DEBUG
                            printf(" Info : %d  given %s ",*((int *) lst->DataPtr),
                                                          YESNO_STR(lst->ArgumentGiven));
#endif

                            break;
         case CMD_DOUBLE  : printf("-%-5s  <double>   %s",lst->CmdLetter,lst->HelpTxt);
#ifdef DEBUG
                            printf(" Info : %g  given %s ",*((double *) lst->DataPtr),
                                                          YESNO_STR(lst->ArgumentGiven));
#endif
                            break;
         case CMD_STRING  : printf("-%-5s \"<string>\"  %s",lst->CmdLetter,lst->HelpTxt);
#ifdef DEBUG
                            printf(" Info : \"%s\"  given %s ",(char *) lst->DataPtr,
                                                          YESNO_STR(lst->ArgumentGiven));
#endif
        } /* switch */
       lst=lst->next;
       printf("\n");
      }
  }

int AnyCommandGiven(CmdArgLst *lst)
/*********************************************************************
 After the call of ScanCommandline this function returns TRUE when any
 command line argument is set by the user otherwise it returns 
FALSE.
**********************************************************************/
  { int any=FALSE;

    while (!(any=any || lst->ArgumentGiven))
       if (!(lst=lst->next))
         break;
    return any;
   }

CmdArgLst *ScanCommandline(CmdArgLst *lst, int argc, char *argv[])
/********************************************************************
This is the true working horse - after the declaration of all possible
command line switches ScanCommandline assign the given arguments and
switches due to the lst. argc and argv are the same as in the head of main.
*********************************************************************/
   { CmdArgLst *head;
     char *sw;
     int i,found;
     int n;
     double d;
     char *s;

     head=lst;
     while(lst) {
       sw=(char *) malloc((strlen(lst->CmdLetter)+2)*sizeof(char));
       strcpy(sw+1,lst->CmdLetter);
       *sw='-';
       for (i=1,found=FALSE; i<argc; i++) { /* loop over arguments */
         if (lst->What != CMD_SWITCH) 
           found=(strcmp(sw,argv[i])==0);
         else {
           if (strcmp(sw,argv[i])==0) { 
             lst->ArgumentGiven=TRUE;
             *((int *) lst->DataPtr) = FALSE;
             break;
            }
           *sw='+';
           if (strcmp(sw,argv[i])==0) {
             lst->ArgumentGiven=TRUE;
             *((int *) lst->DataPtr) = TRUE;
             break;
            }
           *sw='-';
          }  
         if (found && ((i+1)<argc)) {
           switch (lst->What) {
               case CMD_INTEGER : if (sscanf(argv[i+1],"%d",&n)>0) {
                                    *((int *) lst->DataPtr)=n;
                                    lst->ArgumentGiven=TRUE;
                                   }
                                  break;
                case CMD_DOUBLE : if (sscanf(argv[i+1],"%lg",&d)>0) {
                                    *((double *) lst->DataPtr)=d;
                                    lst->ArgumentGiven=TRUE;
                                   }
                                  break;
                case CMD_STRING : strcpy((char *) lst->DataPtr,argv[i+1]);
                                  lst->ArgumentGiven=TRUE;
                                  break; 
            } /* of switch    */
           break;
          }   /* if found ..  */
        }     /* of for       */
       free(sw);
       lst=lst->next;
      }       /* while        */
    return head;
   }


int CmdArgGiven(char *cmd_letter, CmdArgLst *lst)
/* Ask for a certain switch cmd_letter if it is given.
   CmdArgGiven can only called when ScanCommandline
   is finished. */
  {     
    while(lst) {
      if(strcmp(cmd_letter,lst->CmdLetter)==0)
        return lst->ArgumentGiven;
      lst=lst->next;
     }
    return 0;
  }

CmdArgLst *RemoveCmdArgs(CmdArgLst *lst)
/************************************************************
Destroy the complede list of defined command line switches,
is is typical used when the command line is scanned and 
the memory used by lst can be returned to the system.
*************************************************************/
  { CmdArgLst *tail;

    while(lst) {
      tail=lst->next;
      free(lst->CmdLetter);
      free(lst->HelpTxt);
      free(lst);
      lst=tail;
     }
    return NULL;
  }

#undef NEW
#undef YESNO_STR
/***********************************************************
end of cmdln.c
************************************************************/
